require "uu/operation_registry"
require "uu/os/cmd/command_client"
require "uu/os/security/session"

module UU
  module OperationRegistry

    # AppShare allows to share one AppDeployment among multiple tenants, allowing to
    # install an deployment to one AppDeployment (belonging to the tenant which
    # has contracted capacity for the AppDeployment) and share it with other
    # tenants.
    class AppShare < BaseMethods

      # Remote service path.
      # @private
      PATH = "uu-operation-registry/AppShare"

      # Creates a new instance of +AppShare+.
      #
      # @param session [UU::OS::Security::Session] Session to use for remote calls.
      def initialize(session = nil)
        super
      end

      # @!method create(app_deployment_uri, attributes)
      #   Creates a new app share.
      #   @param app_deployment_uri [String, UU::OS::UESURI] App deployment URI.
      #   @param attributes [Hash] AppShare attributes.
      #   @option attributes [String] name AppShare name. May be +nil+.
      #   @option attributes [String] code AppShare code. May be +nil+.
      #   @option attributes [UU::OS::UESURI] tenantUri URI of the primary tenant
      #     (the "paying tenant").
      #   @option attributes [String] urlPath URL path ("/vendor-app") used in
      #     gateway routing data.
      #   @return [UU::OS::UESURI] UESURI of the created app share.

      # @!method self.create(app_deployment_uri, attributes)
      #   Creates a new app share. This method is a convenience shortcut for
      #   the {#create} instance method.
      #   @param app_deployment_uri [String, UU::OS::UESURI] App deployment URI.
      #   @param attributes [Hash] AppShare attributes.
      #   @return [UU::OS::UESURI] UESURI of the created app share.

      # @!method get_attributes(app_share_uri)
      #   Gets the attributes of the app share specified by app share URI
      #   and returns its detail.
      #   @param app_share_uri [String, UU::OS::UESURI] AppShare URI.
      #   @return [Hash] AppShare attributes.

      # @!method self.get_attributes(app_share_uri)
      #   Gets the attributes of the app share specified by app share URI
      #   and returns its detail. This method is a convenience shortcut for
      #   the {#get_attributes} instance method.
      #   @param app_share_uri [String, UU::OS::UESURI] AppShare URI.
      #   @return [Hash] AppShare attributes.

      # @!method set_attributes(app_share_uri, attributes)
      #   Sets attributes of an app share.
      #   @param app_share_uri [String, UU::OS::UESURI] AppShare URI.
      #   @param attributes [Hash] AppShare attributes.
      #   @option attributes [String] name AppShare name. May be +nil+.
      #   @option attributes [String] code AppShare code. May be +nil+.
      #   @return [UU::OS::UESURI] UESURI of the updated app share.

      # @!method self.set_attributes(app_share_uri, attributes)
      #   Sets attributes of an app share. This method is a convenience
      #   shortcut for the {#set_attributes} instance method.
      #   @param app_share_uri [String, UU::OS::UESURI] AppShare URI.
      #   @return [UU::OS::UESURI] UESURI of the updated app share.

      # @!method delete(app_share_uri)
      #   Deletes the specified app share from the registry.
      #   @param app_share_uri [String, UU::OS::UESURI] AppShare URI.
      #   @return [void]

      # @!method self.delete(app_share_uri)
      #   Deletes the specified app share from the registry. This method is a
      #   convenience shortcut for the #{delete} instance method.
      #   @param app_share_uri [String, UU::OS::UESURI] AppShare URI.
      #   @return [void]

      # Returns a list of app shares for the given app deployment. Command does
      # not return all app shares, only a sublist. The list of returned app shares
      # could be filtered using {https://docs.mongodb.org/manual/tutorial/query-documents/
      # Mongo Query}.
      #
      # @param app_deployment_uri [String, UU::OS::UESURI] app deployment URI.
      # @param criteria [Hash] Criteria for listing app shares.
      # @option criteria [String] query Mongo Query to filter the result set.
      #   See {https://docs.mongodb.org/manual/tutorial/query-documents/ Mongo Query tutorial}.
      #   Allowed query filter document fields: code, tenantUri, urlPath, appDeploymentUri
      # @option criteria [Fixnum] pageIndex Result set page index.
      # @option criteria [Fixnum] pageSize Result set page size.
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def get_app_share_list(app_deployment_uri, criteria = {})
        @client.invoke(:getAppShareList, app_deployment_uri, parameters: criteria)
      end

      # Returns list of app shares. This method is a
      # convenience shortcut for the {#get_app_share_list} instance method.
      #
      # @see #get_app_share_list
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def self.get_app_share_list(app_deployment_uri, criteria = {})
        self.new(UU::OS::Security::Session.current_session).get_app_share_list(app_deployment_uri, criteria)
      end

      # Creates multiple shares in one command.
      #
      # @see .bulk_create
      # @param app_deployment_uri [String, UU::OS::UESURI] app deployment URI.
      # @param attributes [Hash] Attributes for bulk create.
      # @option attributes [String] urlPath URL path ("/vendor-app") used in gateway routing data.
      # @option attributes [Array<Hash>] shares Array of hashes. Maximum is 1000 shares per command.
      #   Keys:
      #   * name (String) AppShare name. May be +nil+.
      #   * code (String) AppShare code. May be +nil+.
      #   * tenantUri URI(String, UU::OS::UESURI) UESURI of the tenant. Required.
      # @return [Array] Array of UESURIs of created AppShares.
      def bulk_create(app_deployment_uri, attributes)
        @client.invoke(:bulkCreate, app_deployment_uri, parameters: attributes)
      end

      # Creates multiple shares in one command.
      #
      # @example
      #  require 'uu_operation_registry'
      #
      #  UU::OS::Security::Session.login('')
      #  app_deployment_uri = 'ues:TERR:POOL:APP'
      #  shares = [{:tenantUri => 'ues:TERR:TENANT1'},{:tenantUri => 'ues:TERR:TENANT2'}]
      #  UU::OperationRegistry::AppShare.bulk_create(app_deployment_uri, :urlPath => '/url-path', :shares => shares)
      #
      # @param app_deployment_uri [String, UU::OS::UESURI] app deployment URI.
      # @param attributes [Hash] Attributes for bulk create.
      # @option attributes [String] urlPath URL path ("/vendor-app") used in gateway routing data.
      # @option attributes [Array<Hash>] shares Array of hashes. Maximum is 1000 shares per command.
      #   Keys:
      #   * name (String) AppShare name. May be +nil+.
      #   * code (String) AppShare code. May be +nil+.
      #   * tenantUri URI(String, UU::OS::UESURI) UESURI of the tenant. Required.
      # @return [Array] Array of UESURIs of created AppShares.
      def self.bulk_create(app_deployment_uri, attributes)
        self.new(UU::OS::Security::Session.current_session).bulk_create(app_deployment_uri, attributes)
      end

      # Removes multiple shares in one command.
      #
      # @see .bulk_delete
      # @param app_deployment_uri [String, UU::OS::UESURI] app deployment URI.
      # @param attributes [Hash] Attributes for bulk delete.
      # @option attributes [Array<String, UU::OS::UESURI>] appShares Array of UESURIs of AppShares. Maximum is 1000 appShares per command.
      # @return [Array] Array of UESURIs of removed AppShares.
      def bulk_delete(app_deployment_uri, attributes)
        @client.invoke(:bulkDelete, app_deployment_uri, parameters: attributes)
      end


      # Removes multiple shares in one command.
      #
      # @example
      #  require 'uu_operation_registry'
      #  UU::OS::Security::Session.login('')
      #  app_deployment_uri = 'ues:TERR:POOL:APP'
      #  shares = ['ues:TERR:APPSHARE1', 'ues:TERR:APPSHARE2']
      #  UU::OperationRegistry::AppShare.bulk_delete(app_deployment_uri,  :appShares => shares)
      #
      # @param app_deployment_uri [String, UU::OS::UESURI] app deployment URI.
      # @param attributes [Hash] Attributes for bulk delete.
      # @option attributes [Array<String, UU::OS::UESURI>] appShares Array of UESURIs of AppShares. Maximum is 1000 appShares per command.
      # @return [Array] Array of UESURIs of removed AppShares.
      def self.bulk_delete(app_deployment_uri, attributes)
        self.new(UU::OS::Security::Session.current_session).bulk_delete(app_deployment_uri, attributes)
      end

    end

  end
end
