require "uu/operation_registry"
require "uu/os/cmd/command_client"
require "uu/os/security/session"

module UU
  module OperationRegistry

    # Cloud.
    class Cloud < BaseMethods

      # Remote service path.
      # @private
      PATH = "uu-operation-registry/Cloud"

      # Creates a new instance of +Cloud+.
      #
      # @param session [UU::OS::Security::Session] Session to use for remote calls.
      def initialize(session = nil)
        super
      end

      # @!method get_attributes(cloud_uri)
      #   Gets the attributes of the cloud specified by cloud URI
      #   and returns its detail.
      #   @param cloud_uri [String, UU::OS::UESURI] Cloud URI.
      #   @return [Hash] Cloud attributes.
      #
      #   @example Response
      #     {
      #       :name => "Plus4U Cloud",
      #       :code => "PLUS4U_CLOUD",
      #       :operTerOid => "0",
      #       :operTerCode => "OT-BT",
      #       :env => "{\"uuClientProperties\":{\"uu.server-url\":\"https://api.plus4u.net\",\"uu.c3.base_url\":\"https://sys.plus4u.net\",\"uu.c3.append_routing_data\":\"false\",\"uu.operation.registry.base_url\":\"https://sys.plus4u.net\",\"uu.operation.registry.append_routing_data\":\"false\"},\"gemrc\":{\":backtrace\":\"false\",\":benchmark\":\"false\",\":bulk_threshold\":\"100\",\":sources\":[\"http://rubygems.org\",\"http://gems.plus4u.net\"],\":update_sources\":\"true\",\":verbose\":\"false\",\"gem\":\"--env-shebang --no-doc --no-ri --no-document\"}}",
      #       :uri => "ues:OT-BT[0]:PLUS4U_CLOUD[58ea977fab27248ac0d96d36]"
      #     }

      # @!method self.get_attributes(cloud_uri)
      #   Gets the attributes of the cloud specified by cloud URI
      #   and returns its detail. This method is a convenience shortcut for
      #   the {#get_attributes} instance method.
      #   @param cloud_uri [String, UU::OS::UESURI] Cloud URI.
      #   @return [Hash] Cloud attributes.

      # Sets attributes of a Cloud.
      #
      # @example Set env JSON
      #  require 'uu_operation_registry'
      #  require 'json'
      #
      #  UU::OS::Security::Session.login('...')
      #
      #  env = {
      #    "uuClientProperties" => {
      #      "uu.server-url" => "https://api.plus4u.net",
      #      "uu.c3.base_url" => "https://sys.plus4u.net",
      #      "uu.c3.append_routing_data" => "false",
      #      "uu.operation.registry.base_url" => "https://sys.plus4u.net",
      #      "uu.operation.registry.append_routing_data" => "false"
      #    },
      #    "gemrc" => {
      #      ":backtrace" => "false",
      #      ":benchmark" => "false",
      #      ":bulk_threshold" => "100",
      #      ":sources" => [
      #        "http://rubygems.org",
      #        "http://gems.plus4u.net"
      #      ],
      #      ":update_sources" => "true",
      #      ":verbose" => "false",
      #      "gem" => "--env-shebang --no-doc --no-ri --no-document"
      #    }
      #  }
      #
      #  UU::OperationRegistry::Cloud.set_attributes(
      #    'ues:CLOUD:URI',
      #    :env => env.to_json
      #  )
      #
      # @param cloud_uri [String, UU::OS::UESURI] Cloud URI.
      # @param attributes [Hash] Cloud attributes.
      # @option attributes [String] env Env configuration JSON. Contains
      #   configuration (gemrc, uu-client.properties) for nodes deployed in the
      #   context of the Cloud.  May be +nil+.
      # @return [UU::OS::UESURI] UESURI of the updated Cloud.
      def set_attributes(cloud_uri, attributes)
        @client.invoke(:setAttributes, cloud_uri, parameters: attributes)
      end

      # Sets attributes of a Cloud. This method is a convenience
      # shortcut for the {#set_attributes} instance method.
      #
      # @param cloud_uri [String, UU::OS::UESURI] Cloud URI.
      # @return [UU::OS::UESURI] UESURI of the updated Cloud.
      def self.set_attributes(cloud_uri, attributes)
        self.new(UU::OS::Security::Session.current_session).set_attributes(cloud_uri, attributes)
      end

      # Returns a list of containers in the given cloud. Command does
      # not return all containers, only a sublist. The list of returned containers
      # could be filtered using {https://docs.mongodb.org/manual/tutorial/query-documents/
      # Mongo Query}.
      #
      # @param cloud_uri [String, UU::OS::UESURI] Cloud URI.
      # @param criteria [Hash] Criteria for listing containers.
      # @option criteria [String] query Mongo Query to filter the result set.
      #   See {https://docs.mongodb.org/manual/tutorial/query-documents/ Mongo Query tutorial}.
      #   Allowed query filter document fields: name, code, nodeImageUri, nodeUri, nodeSetUri, state
      # @option criteria [Fixnum] pageIndex Result set page index.
      # @option criteria [Fixnum] pageSize Result set page size.
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def get_container_list(cloud_uri, criteria = {})
        @client.invoke(:getContainerList, cloud_uri, parameters: criteria)
      end

      # Returns list of containers. This method is a
      # convenience shortcut for the {#get_container_list} instance method.
      #
      # @see #get_container_list
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def self.get_container_list(cloud_uri, criteria = {})
        self.new(UU::OS::Security::Session.current_session).get_container_list(cloud_uri, criteria)
      end

      # Returns a list of resource groups in the given cloud. Command does
      # not return all resource groups, only a sublist. The list of returned resource groups
      # could be filtered using {https://docs.mongodb.org/manual/tutorial/query-documents/
      # Mongo Query}.
      #
      # @param cloud_uri [String, UU::OS::UESURI] Cloud URI.
      # @param criteria [Hash] Criteria for listing resource groups.
      # @option criteria [String] query Mongo Query to filter the result set.
      #   See {https://docs.mongodb.org/manual/tutorial/query-documents/ Mongo Query tutorial}.
      #   Allowed query filter document fields: code, regionUri
      # @option criteria [Fixnum] pageIndex Result set page index.
      # @option criteria [Fixnum] pageSize Result set page size.
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def get_resource_group_list(cloud_uri, criteria = {})
        @client.invoke(:getResourceGroupList, cloud_uri, parameters: criteria)
      end

      # Returns list of resource groups. This method is a
      # convenience shortcut for the {#get_resource_group_list} instance method.
      #
      # @see #get_resource_group_list
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def self.get_resource_group_list(cloud_uri, criteria = {})
        self.new(UU::OS::Security::Session.current_session).get_resource_group_list(cloud_uri, criteria)
      end

      # Returns a list of statistics for resource groups in the given cloud.
      # The list of returned statistics could be filtered using
      # {https://docs.mongodb.org/manual/tutorial/query-documents/ Mongo Query}.
      #
      # @example Response
      #
      #   {
      #     :totalSize => 2,
      #     :criteria => {
      #       :pageIndex => 0,
      #       :pageSize => 1000
      #     },
      #     :pageEntries => [
      #       {
      #         :groupUri => "ues:[99923616732452117]:[59cde284f8458af04c9c5541]",
      #         :name => "System Resource Group",
      #         :description => "Operation of system services",
      #         :stats => {
      #           :total => {
      #             :cpu => 96,
      #             :mem => 72000,
      #             :storage => 120000
      #           },
      #           :free => {
      #             :cpu => 75,
      #             :mem => 61500,
      #             :storage => 99000
      #           },
      #           :availableNodes => {
      #             :G1_S => 75,
      #             :G1_M => 60,
      #             :G1_L => 29,
      #             :G1_XL => 10,
      #             :C1_S => 37,
      #             :C1_M => 18,
      #             :C1_L => 8
      #           }
      #         }
      #       },
      #       {
      #        :groupUri => "ues:[99923616732452117]:[59cde288f8458af49d9c5543]",
      #        :name => "uuApps Resource Group",
      #        :description => "Operation of uuApps for uuOS 8",
      #        :stats => {
      #          :total => {
      #            :cpu => 32,
      #            :mem => 24000,
      #            :storage => 40000
      #           },
      #          :free => {
      #            :cpu => 25,
      #            :mem => 20500,
      #            :storage => 33000
      #           },
      #          :availableNodes => {
      #            :G1_S => 25,
      #            :G1_M => 20,
      #            :G1_L => 10,
      #            :G1_XL => 4,
      #            :C1_S => 12,
      #            :C1_M => 6,
      #            :C1_L => 3
      #           }
      #         }
      #       }
      #     ]
      #   }
      #
      # @note This command returns canonical form of ResourceGroup UESURI (i.e. UESURI containing only IDs, not codes).
      #
      # @param cloud_uri [String, UU::OS::UESURI] Cloud URI.
      # @param criteria [Hash] Criteria for listing resource groups.
      # @option criteria [String] query Mongo Query to filter the result set.
      #   See {https://docs.mongodb.org/manual/tutorial/query-documents/ Mongo Query tutorial}.
      #   Allowed query filter document fields: code, name, regionUri
      # @option criteria [Fixnum] pageIndex Result set page index.
      # @option criteria [Fixnum] pageSize Result set page size.
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def get_resource_group_stats(cloud_uri, criteria = {})
        @client.invoke(:getResourceGroupStats, cloud_uri, parameters: criteria)
      end

      # Returns a list of statistics for resource groups in the given cloud.
      # This method is a convenience shortcut for the {#get_resource_group_stats} instance method.
      #
      # @see #get_resource_group_stats
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def self.get_resource_group_stats(cloud_uri, criteria = {})
        self.new(UU::OS::Security::Session.current_session).get_resource_group_stats(cloud_uri, criteria)
      end

      # Returns a list of resource pools in the given cloud. Command does
      # not return all resource pools, only a sublist. The list of returned resource pools
      # could be filtered using {https://docs.mongodb.org/manual/tutorial/query-documents/
      # Mongo Query}.
      #
      # @param cloud_uri [String, UU::OS::UESURI] Cloud URI.
      # @param criteria [Hash] Criteria for listing resource pools.
      # @option criteria [String] query Mongo Query to filter the result set.
      #   See {https://docs.mongodb.org/manual/tutorial/query-documents/ Mongo Query tutorial}.
      #   Allowed query filter document fields: code, regionUri
      # @option criteria [Fixnum] pageIndex Result set page index.
      # @option criteria [Fixnum] pageSize Result set page size.
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def get_resource_pool_list(cloud_uri, criteria = {})
        @client.invoke(:getResourcePoolList, cloud_uri, parameters: criteria)
      end

      # Returns list of resource pools. This method is a
      # convenience shortcut for the {#get_resource_pool_list} instance method.
      #
      # @see #get_resource_pool_list
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def self.get_resource_pool_list(cloud_uri, criteria = {})
        self.new(UU::OS::Security::Session.current_session).get_resource_pool_list(cloud_uri, criteria)
      end

      # Returns a list of hosts in the given cloud. Command does
      # not return all hosts, only a sublist. The list of returned hosts
      # could be filtered using {https://docs.mongodb.org/manual/tutorial/query-documents/
      # Mongo Query}.
      #
      # @param cloud_uri [String, UU::OS::UESURI] Cloud URI.
      # @param criteria [Hash] Criteria for listing hosts.
      # @option criteria [String] query Mongo Query to filter the result set.
      #   See {https://docs.mongodb.org/manual/tutorial/query-documents/ Mongo Query tutorial}.
      #   Allowed query filter document fields:
      #   code, state, hostname, resourceGroupUri, totalCapacity, freeCapacity
      # @option criteria [Fixnum] pageIndex Result set page index.
      # @option criteria [Fixnum] pageSize Result set page size.
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def get_host_list(cloud_uri, criteria = {})
        @client.invoke(:getHostList, cloud_uri, parameters: criteria)
      end

      # Returns list of hosts. This method is a
      # convenience shortcut for the {#get_host_list} instance method.
      #
      # @see #get_host_list
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def self.get_host_list(cloud_uri, criteria = {})
        self.new(UU::OS::Security::Session.current_session).get_host_list(cloud_uri, criteria)
      end

      # Returns a list of message busses in the given cloud. Command does
      # not return all message busses, only a sublist. The list of returned message busses
      # could be filtered using {https://docs.mongodb.org/manual/tutorial/query-documents/
      # Mongo Query}.
      #
      # @param cloud_uri [String, UU::OS::UESURI] Cloud URI.
      # @param criteria [Hash] Criteria for listing message busses.
      # @option criteria [String] query Mongo Query to filter the result set.
      #   See {https://docs.mongodb.org/manual/tutorial/query-documents/ Mongo Query tutorial}.
      #   Allowed query filter document fields:
      #   code, repositoryUri, tenantUri, queuePairsLimit, state, labels
      # @option criteria [Fixnum] pageIndex Result set page index.
      # @option criteria [Fixnum] pageSize Result set page size.
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def get_msg_bus_list(cloud_uri, criteria = {})
        @client.invoke(:getMsgBusList, cloud_uri, parameters: criteria)
      end

      # Returns list of message busses. This method is a
      # convenience shortcut for the {#get_msg_bus_list} instance method.
      #
      # @see #get_msg_bus_list
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def self.get_msg_bus_list(cloud_uri, criteria = {})
        self.new(UU::OS::Security::Session.current_session).get_msg_bus_list(cloud_uri, criteria)
      end

      # Returns a list of nodes in the given cloud. Command does
      # not return all nodes, only a sublist. The list of returned nodes
      # could be filtered using {https://docs.mongodb.org/manual/tutorial/query-documents/
      # Mongo Query}.
      #
      # @param cloud_uri [String, UU::OS::UESURI] Cloud URI.
      # @param criteria [Hash] Criteria for listing nodes.
      # @option criteria [String] query Mongo Query to filter the result set.
      #   See {https://docs.mongodb.org/manual/tutorial/query-documents/ Mongo Query tutorial}.
      #   Allowed query filter document fields: code, nodeSizeUri, hostUri, resourcePoolUri
      # @option criteria [Fixnum] pageIndex Result set page index.
      # @option criteria [Fixnum] pageSize Result set page size.
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def get_node_list(cloud_uri, criteria = {})
        @client.invoke(:getNodeList, cloud_uri, parameters: criteria)
      end

      # Returns list of nodes. This method is a
      # convenience shortcut for the {#get_node_list} instance method.
      #
      # @see #get_node_list
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def self.get_node_list(cloud_uri, criteria = {})
        self.new(UU::OS::Security::Session.current_session).get_node_list(cloud_uri, criteria)
      end

      # Returns list of app deployments in the given cloud. Command does
      # not return all nodes, only a sublist. The list of returned nodes
      # could be filtered using {https://docs.mongodb.org/manual/tutorial/query-documents/
      # Mongo Query}.
      #
      # @param cloud_uri [String, UU::OS::UESURI] Cloud URI.
      # @param criteria [Hash] Criteria for listing app deployments.
      # @option criteria [String] query Mongo Query to filter the result set.
      #   See {https://docs.mongodb.org/manual/tutorial/query-documents/ Mongo Query tutorial}.
      #   Allowed query filter document fields: code, appBoxUri, resourcePoolUri, tenantUri, state, urlPath
      # @option criteria [Fixnum] pageIndex Result set page index.
      # @option criteria [Fixnum] pageSize Result set page size.
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def get_app_deployment_list(cloud_uri, criteria = {})
        @client.invoke(:getAppDeploymentList, cloud_uri, parameters: criteria)
      end

      # Returns list of app deployments. This method is a
      # convenience shortcut for the {#get_app_deployment_list} instance method.
      #
      # @see #get_app_deployment_list
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def self.get_app_deployment_list(cloud_uri, criteria = {})
        self.new(UU::OS::Security::Session.current_session).get_app_deployment_list(cloud_uri, criteria)
      end


      # Returns list of app shares in the given cloud. Command does
      # not return all nodes, only a sublist. The list of returned nodes
      # could be filtered using {https://docs.mongodb.org/manual/tutorial/query-documents/
      # Mongo Query}.
      #
      # @param cloud_uri [String, UU::OS::UESURI] Cloud URI.
      # @param criteria [Hash] Criteria for listing app shares.
      # @option criteria [String] query Mongo Query to filter the result set.
      #   See {https://docs.mongodb.org/manual/tutorial/query-documents/ Mongo Query tutorial}.
      #   Allowed query filter document fields: code, tenantUri, urlPath, appDeploymentUri
      # @option criteria [Fixnum] pageIndex Result set page index.
      # @option criteria [Fixnum] pageSize Result set page size.
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def get_app_share_list(cloud_uri, criteria = {})
        @client.invoke(:getAppShareList, cloud_uri, parameters: criteria)
      end

      # Returns list of app shares. This method is a
      # convenience shortcut for the {#get_app_share_list} instance method.
      #
      # @see #get_app_share_list
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def self.get_app_share_list(cloud_uri, criteria = {})
        self.new(UU::OS::Security::Session.current_session).get_app_share_list(cloud_uri, criteria)
      end

      # Returns list of node sets in the given cloud. Command does
      # not return all nodes, only a sublist. The list of returned nodes
      # could be filtered using {https://docs.mongodb.org/manual/tutorial/query-documents/
      # Mongo Query}.
      #
      # @note Attributes "nodeSizeUri", "nodeCount" and "inboundPorts" are set only for ResourceGroup of type "SWARM".
      # @param cloud_uri [String, UU::OS::UESURI] Cloud URI.
      # @param criteria [Hash] Criteria for listing node sets.
      # @option criteria [String] query Mongo Query to filter the result set.
      #   See {https://docs.mongodb.org/manual/tutorial/query-documents/ Mongo Query tutorial}.
      #   Allowed query filter document fields: code, tenantUri, urlPath, appDeploymentUri
      # @option criteria [Fixnum] pageIndex Result set page index.
      # @option criteria [Fixnum] pageSize Result set page size.
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def get_node_set_list(cloud_uri, criteria = {})
        @client.invoke(:getNodeSetList, cloud_uri, parameters: criteria)
      end

      # Returns list of node sets. This method is a
      # convenience shortcut for the {#get_node_set_list} instance method.
      #
      # @see #get_node_set_list
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def self.get_node_set_list(cloud_uri, criteria = {})
        self.new(UU::OS::Security::Session.current_session).get_node_set_list(cloud_uri, criteria)
      end

    end

  end
end
