require "uu/operation_registry"
require "uu/os/cmd/command_client"
require "uu/os/security/session"

module UU
  module OperationRegistry

    # Message bus is a logical set of queues.
    class MessageBus < BaseMethods

      # Remote service path.
      # @private
      PATH = "uu-operation-registry/MessageBus"

      # Creates a new instance of +MessageBus+.
      #
      # @param session [UU::OS::Security::Session] Session to use for remote calls.
      def initialize(session = nil)
        super
      end

      # @!method create(msg_bus_repository_uri, attributes)
      #   Creates a new message bus.
      #   @param msg_bus_repository_uri [String, UU::OS::UESURI] Message bus
      #     repository URI.
      #   @param attributes [Hash] Message bus attributes.
      #   @option attributes [String] name Message bus name. May be +nil+.
      #   @option attributes [String] code Message bus code. May be +nil+.
      #   @option attributes [Array] labels Array of message bus labels. 
      #     Supported labels are: DEDICATED
      #     May be +nil+.
      #   @option attributes [String] state Message bus state. 
      #     One from {UU::OperationRegistry::MessageBusState}.
      #     May be +nil+.
      #   @option attributes [Fixnum] queuePairsLimit Message bus capacity.
      #     May be +nil+.
      #   @option attributes [String] description Message bus description.
      #     May be +nil+.
      #   @option attributes [String] executorNodeSize Node size of executor
      #     container. May be +nil+. Default node size defined in the given
      #     message bus repository is used if not specified.
      #   @return [UU::OS::UESURI] UESURI of the created message bus.

      # @!method self.create(msg_bus_repository_uri, attributes)
      #   Creates a new message bus. This method is a convenience shortcut for
      #   the {#create} instance method.
      #   @param msg_bus_repository_uri [String, UU::OS::UESURI] Message bus
      #     repository URI.
      #   @param attributes [Hash] Message bus attributes.
      #   @return [UU::OS::UESURI] UESURI of the created message bus.

      # @!method get_attributes(msg_bus_uri)
      #   Gets the attributes of the message bus specified by message bus URI
      #   and returns its detail.
      #   @param msg_bus_uri [String, UU::OS::UESURI] Message bus URI.
      #   @return [Hash] Message bus attributes.

      # @!method self.get_attributes(msg_bus_uri)
      #   Gets the attributes of the message bus specified by message bus URI
      #   and returns its detail. This method is a convenience shortcut for
      #   the {#get_attributes} instance method.
      #   @param msg_bus_uri [String, UU::OS::UESURI] Message bus URI.
      #   @return [Hash] Message bus attributes.

      # @!method set_attributes(msg_bus_uri, attributes)
      #   Sets attributes of a message bus.
      #   @param msg_bus_uri [String, UU::OS::UESURI] Message bus URI.
      #   @param attributes [Hash] Message bus attributes.
      #   @option attributes [String] name Message bus name. May be +nil+.
      #   @option attributes [String] code Message bus code. May be +nil+.
      #   @option attributes [Array] labels Array of message bus labels. 
      #     Supported labels are: DEDICATED
      #     May be +nil+.
      #   @option attributes [String] state Message bus state. 
      #     One from {UU::OperationRegistry::MessageBusState}.
      #     May be +nil+.
      #   @option attributes [String] description Message bus description.
      #     May be +nil+.
      #   @option attributes [String] executorNodeSize Node size of executor
      #     container. May be +nil+. 
      #   @return [UU::OS::UESURI] UESURI of the updated message bus.

      # @!method self.set_attributes(msg_bus_uri, attributes)
      #   Sets attributes of a message bus. This method is a convenience
      #   shortcut for the {#set_attributes} instance method.
      #   @param msg_bus_uri [String, UU::OS::UESURI] Message bus URI.
      #   @return [UU::OS::UESURI] UESURI of the updated message bus.

      # @!method delete(msg_bus_uri)
      #   Deletes the specified message bus from the registry.
      #   @param msg_bus_uri [String, UU::OS::UESURI] Message bus URI.
      #   @return [void]

      # @!method self.delete(msg_bus_uri)
      #   Deletes the specified message bus from the registry. This method is a
      #   convenience shortcut for the #{delete} instance method.
      #   @param msg_bus_uri [String, UU::OS::UESURI] Message bus URI.
      #   @return [void]

      # Returns a list of message buses in the given message bus repository.
      # Command does not return all message buses, only a sublist. The list of
      # returned message buses could be filtered using
      # {https://docs.mongodb.org/manual/tutorial/query-documents/
      # Mongo Query}.
      #
      # @param msg_bus_repository_uri [String, UU::OS::UESURI] Message bus
      #   repository URI.
      # @param criteria [Hash] Criteria for listing message buses.
      # @option criteria [String] query Mongo Query to filter the result set.
      #   See {https://docs.mongodb.org/manual/tutorial/query-documents/
      #   Mongo Query tutorial}. 
      #   Allowed query filter document fields: code, repositoryUri, tenantUri 
      # @option criteria [Fixnum] pageIndex Result set page index.
      # @option criteria [Fixnum] pageSize Result set page size.
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def get_message_bus_list(msg_bus_repository_uri, criteria = {})
        @client.invoke(:getMessageBusList, msg_bus_repository_uri, parameters: criteria)
      end

      # Returns list of message buses. This method is a
      # convenience shortcut for the {#get_message_bus_list} instance method.
      #
      # @see #get_message_bus_list
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def self.get_message_bus_list(msg_bus_repository_uri, criteria = {})
        self.new(UU::OS::Security::Session.current_session).get_message_bus_list(msg_bus_repository_uri, criteria)
      end

      # Allocates new queue pairs.
      #
      # @param msg_bus_repository_uri [String, UU::OS::UESURI] Message bus
      #   repository URI.
      # @param attributes [Hash] Attributes.
      # @option attributes [Fixnum] count
      # @option attributes [String, UU::OS::UESURI] appDeploymentUri UESURI of
      #   deployment that this message bus belongs to.
      # @return [Array<String>]
      def allocate_queue_pairs(msg_bus_repository_uri, attributes = {})
        @client.invoke(:allocateQueuePairs, msg_bus_repository_uri, parameters: attributes)
      end

      # Allocates new queue pairs.
      # This method is a convenience shortcut for the {#allocate_queue_pairs} instance method.
      #
      # @see #allocate_queue_pairs
      # @return [Array<String>]
      def self.allocate_queue_pairs(msg_bus_repository_uri, attributes = {})
        self.new(UU::OS::Security::Session.current_session).allocate_queue_pairs(msg_bus_repository_uri, attributes)
      end

      # Releases allocated queue pairs.
      #
      # @param msg_bus_repository_uri [String, UU::OS::UESURI] Message bus
      #   repository URI.
      # @param attributes [Hash] Attributes.
      # @option attributes [Fixnum] count
      # @option attributes [String, UU::OS::UESURI] appDeploymentUri UESURI of
      #   deployment that this message bus belongs to.
      # @return [void]
      def release_queue_pairs(msg_bus_repository_uri, attributes = {})
        @client.invoke(:releaseQueuePairs, msg_bus_repository_uri, parameters: attributes)
      end

      # Releases allocated queue pairs.
      # This method is a convenience shortcut for the {#release_queue_pairs} instance method.
      #
      # @see #release_queue_pairs
      # @return [void]
      def self.release_queue_pairs(msg_bus_repository_uri, attributes = {})
        self.new(UU::OS::Security::Session.current_session).release_queue_pairs(msg_bus_repository_uri, attributes)
      end


    end

  end
end
