require "uu/operation_registry"
require "uu/os/cmd/command_client"
require "uu/os/security/session"

module UU
  module OperationRegistry

    # Message bus repository represents a cluster of messaging servers.
    class MessageBusRepository < BaseMethods

      # Remote service path.
      # @private
      PATH = "uu-operation-registry/MessageBusRepository"

      # Creates a new instance of +MessageBusRepository+.
      #
      # @param session [UU::OS::Security::Session] Session to use for remote calls.
      def initialize(session = nil)
        super
      end

      # @!method create(resource_pool_uri, attributes)
      #   Creates a new message bus repository.
      #
      #   @example
      #     client = UU::OperationRegistry::MessageBusRepository.new(UU::OS::Security::Session.current_session)
      #     msg_bus_repository_uri = client.create(resource_pool_uri,
      #       name: "Message Bus Repository 01",
      #       code: "MSG_BUS_REPOSITORY_01",
      #       executorImageName: "job_executor_cmd:2.0.0",
      #       executorNodeSize: "G1_M",
      #       dispatcherHostnames: ["10.30.0.163"],
      #       serverHostnames: [{hostname: "10.30.0.163", port: 5302, jmxPort: 12345}],
      #       gatewayHostnames: ["10.30.4.4"],
      #       gatewayUrl: "http://async.internal.plus4u.net",
      #       callbackGatewayUrl: "https://cmd.plus4u.net"
      #     )
      #
      #   @param resource_pool_uri [String, UU::OS::UESURI] Resource pool URI.
      #   @param attributes [Hash] Message bus repository attributes.
      #   @option attributes [String] name Message bus repository name. May be +nil+.
      #   @option attributes [String] code Message bus repository code. May be +nil+.
      #   @option attributes [String] description Message bus repository description.
      #     May be +nil+.
      #   @option attributes [String] executorImageName Name of node image for
      #     executor containers.
      #   @option attributes [String] executorNodeSize Default node size of
      #     executor container. Each message bus may optionally use another node
      #     size if needed.
      #   @option attributes [Array<String>] dispatcherHostnames Host names or IP
      #     addresses of UESAsyncCommandExecutor (Tomcat) servers for this
      #     message bus repository.
      #   @option attributes [Array<Hash>] serverHostnames Addresses and ports
      #     (format is {hostname: +address+, jmxPort: +port+, port: +port+ where
      #     address is either host name or IP) of JBoss servers.
      #   @option attributes [Array<String>] gatewayHostnames Addresses of
      #     Docker hosts where async command gateways for this message bus
      #     repository are deployed.
      #   @option attributes [String] gatewayUrl Base URL of async command
      #     gateway.
      #   @option attributes [String] callbackGatewayUrl Base URL of command
      #     gateway.
      #   @return [UU::OS::UESURI] UESURI of the created message bus repository.

      # @!method self.create(resource_pool_uri, attributes)
      #   Creates a new message bus repository. This method is a convenience
      #   shortcut for the {#create} instance method.
      #   @param resource_pool_uri [String, UU::OS::UESURI] Resource pool URI.
      #   @param attributes [Hash] Message bus repository attributes.
      #   @return [UU::OS::UESURI] UESURI of the created message bus repository.

      # @!method get_attributes(msg_bus_repository_uri)
      #   Gets the attributes of the message bus repository specified by message
      #   bus repository URI and returns its detail.
      #   @param msg_bus_repository_uri [String, UU::OS::UESURI] Message bus
      #     repository URI.
      #   @return [Hash] Message bus repository attributes.

      # @!method self.get_attributes(msg_bus_repository_uri)
      #   Gets the attributes of the message bus repository specified by message
      #   bus repository URI
      #   and returns its detail. This method is a convenience shortcut for
      #   the {#get_attributes} instance method.
      #   @param msg_bus_repository_uri [String, UU::OS::UESURI] Message bus
      #     repository URI.
      #   @return [Hash] Message bus repository attributes.

      # @!method set_attributes(msg_bus_repository_uri, attributes)
      #   Sets attributes of a message bus repository.
      #   @param msg_bus_repository_uri [String, UU::OS::UESURI] Message bus
      #     repository URI.
      #   @param attributes [Hash] Message bus repository attributes.
      #   @option attributes [String] name Message bus repository name.
      #     May be +nil+.
      #   @option attributes [String] code Message bus repository code.
      #     May be +nil+.
      #   @option attributes [String] description Message bus repository
      #     description. May be +nil+.
      #   @option attributes [String] executorImageName Name of node image for
      #     executor containers. May be +nil+.
      #   @option attributes [String] executorNodeSize Default node size of
      #     executor container. Each message bus may optionally use another node
      #     size if needed. May be +nil+.
      #   @option attributes [Array<String>] dispatcherHostnames Host names or IP
      #     addresses of UESAsyncCommandExecutor (Tomcat) servers for this
      #     message bus repository. May be +nil+.
      #   @option attributes [Array<Hash>] serverHostnames Addresses and ports
      #     (format is {hostname: +address+, jmxPort: +port+, port: +port+ where
      #     address is either host name or IP) of JBoss servers.
      #   @option attributes [Array<String>] gatewayHostnames Addresses of
      #     Docker hosts where async command gateways for this message bus
      #     repository are deployed. May be +nil+.
      #   @option attributes [String] gatewayUrl Base URL of async command
      #     gateway. May be +nil+.
      #   @option attributes [String] callbackGatewayUrl Base URL of command
      #     gateway. May be +nil+.
      #   @return [UU::OS::UESURI] UESURI of the updated message bus repository.

      # @!method self.set_attributes(msg_bus_repository_uri, attributes)
      #   Sets attributes of a message bus repository. This method is a
      #   convenience shortcut for the {#set_attributes} instance method.
      #   @param msg_bus_repository_uri [String, UU::OS::UESURI] Message bus
      #     repository URI.
      #   @return [UU::OS::UESURI] UESURI of the updated message bus repository.

      # @!method delete(msg_bus_repository_uri)
      #   Deletes the specified message bus repository from the registry.
      #   @param msg_bus_repository_uri [String, UU::OS::UESURI] Message bus
      #     repository URI.
      #   @return [void]

      # @!method self.delete(msg_bus_repository_uri)
      #   Deletes the specified message bus repository from the registry. This
      #   method is a convenience shortcut for the #{delete} instance method.
      #   @param msg_bus_repository_uri [String, UU::OS::UESURI] Message bus
      #     repository URI.
      #   @return [void]

      # Returns a list of message bus repositories in the given resource pool.
      # Command does not return all message bus repositories, only a sublist.
      # The list of returned message bus repositories could be filtered using
      # {https://docs.mongodb.org/manual/tutorial/query-documents/
      # Mongo Query}.
      #
      # @param resource_pool_uri [String, UU::OS::UESURI] Message bus
      #   repository URI.
      # @param criteria [Hash] Criteria for listing message bus repositories.
      # @option criteria [String] query Mongo Query to filter the result set.
      #   See {https://docs.mongodb.org/manual/tutorial/query-documents/
      #   Mongo Query tutorial}.
      #   Allowed query filter document fields:
      #    code, resourcePoolUri, executorImageName, executorNodeSize,
      #    dispatcherHostnames, serverHostnames, gatewayHostnames, 
      #    gatewayUrl, callbackGatewayUrl, tenantUri
      # @option criteria [Fixnum] pageIndex Result set page index.
      # @option criteria [Fixnum] pageSize Result set page size.
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def get_message_bus_repository_list(resource_pool_uri, criteria = {})
        @client.invoke(:getMessageBusRepositoryList, resource_pool_uri, parameters: criteria)
      end

      # Returns list of message bus repositories. This method is a
      # convenience shortcut for the {#get_message_bus_repository_list} instance method.
      #
      # @see #get_message_bus_repository_list
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def self.get_message_bus_repository_list(resource_pool_uri, criteria = {})
        self.new(UU::OS::Security::Session.current_session).get_message_bus_repository_list(resource_pool_uri, criteria)
      end

    end

  end
end
