require "uu/operation_registry"
require "uu/os/cmd/command_client"
require "uu/os/security/session"

module UU
  module OperationRegistry

    # Node.
    class Node < BaseMethods

      # Remote service path.
      # @private
      PATH = "uu-operation-registry/Node"

      # Creates a new instance of +Node+.
      #
      # @param session [UU::OS::Security::Session] Session to use for remote calls.
      def initialize(session = nil)
        super
      end

      # @!method create(host_uri, attributes)
      #   Creates a new node.
      #   @param host_uri [String, UU::OS::UESURI] Host URI.
      #   @param attributes [Hash] Node attributes.
      #   @option attributes [String] name Node name. May be +nil+.
      #   @option attributes [String] code Node code. May be +nil+.
      #   @option attributes [String] description Node description.
      #     May be +nil+.
      #   @option attributes [UU::OS::UESURI] nodeSizeUri Node size URI.
      #   @option attributes [UU::OS::UESURI] nodeSetUri NodeSet URI.
      #   @option attributes [UU::OS::UESURI] resourcePoolUri Resource pool this node
      #     belongs to.
      #   @option attributes [String] hostname Hostname of the host where the
      #     node is running. May be +nil+.
      #   @option attributes [UU::OS::UESURI] appDeploymentUri UESURI of
      #     deployment that this node belongs to. May be +nil+.
      #   @return [UU::OS::UESURI] UESURI of the created node.

      # @!method self.create(host_uri, attributes)
      #   Creates a new node. This method is a convenience shortcut for
      #   the {#create} instance method.
      #   @param host_uri [String, UU::OS::UESURI] Host URI.
      #   @param attributes [Hash] Node attributes.
      #   @return [UU::OS::UESURI] UESURI of the created node.

      # @!method get_attributes(node_uri)
      #   Gets the attributes of the node specified by node URI
      #   and returns its detail.
      #   @param node_uri [String, UU::OS::UESURI] Node URI.
      #   @return [Hash] Node attributes.

      # @!method self.get_attributes(node_uri)
      #   Gets the attributes of the node specified by node URI
      #   and returns its detail. This method is a convenience shortcut for
      #   the {#get_attributes} instance method.
      #   @param node_uri [String, UU::OS::UESURI] Node URI.
      #   @return [Hash] Node attributes.

      # @!method set_attributes(node_uri, attributes)
      #   Sets attributes of an node.
      #   @param node_uri [String, UU::OS::UESURI] Node URI.
      #   @param attributes [Hash] Node attributes.
      #   @option attributes [String] name Node name. May be +nil+.
      #   @option attributes [String] code Node code. May be +nil+.
      #   @option attributes [String] description Node description.
      #     May be +nil+.
      #   @option attributes [UU::OS::UESURI] nodeSizeUri Node size URI.
      #     May be +nil+.
      #   @option attributes [String] hostname Hostname of the host where the
      #     node is running. May be +nil+.
      #   @option attributes [UU::OS::UESURI] appDeploymentUri UESURI of
      #     deployment that this node belongs to. May be +nil+.
      #   @option attributes [UU::OS::UESURI] nodeSetUri NodeSet URI.
      #   @return [UU::OS::UESURI] UESURI of the updated node.

      # @!method self.set_attributes(node_uri, attributes)
      #   Sets attributes of an node. This method is a convenience
      #   shortcut for the {#set_attributes} instance method.
      #   @param node_uri [String, UU::OS::UESURI] Node URI.
      #   @return [UU::OS::UESURI] UESURI of the updated node.

      # Deletes the specified node from the registry.
      #
      # Node can be successfully deleted if all these conditions are met:
      #
      # 1. no container exists referencing this node (via +nodeUri+)
      # 2. +nodeSizeUri+ on this node is set
      # 3. node size referenced (via +nodeSizeUri+) from this node exists
      # 4. +resourcePoolUri+ on this node is set
      # 5. resource pool referenced (via +resourcePoolUri+) from this node
      #    exists
      #
      # Note: Condition 1 ensures data consistency, conditions 2-5 ensure
      # that a proper capacity log record can be created to terminate
      # billing of the deleted node.
      #
      # Under special circumstances, a node can eventually be deleted even if
      # the afforementioned conditions 2-5 are not met. This is possible setting
      # +force+ parameter to +true+.
      #
      # Note: If condition 1 is not met, the node cannot be deleted, not even
      # if +force+ is +true+.
      #
      # @example Node deletion
      #   require "uu_operation_registry"
      #
      #   UU::OS::Security::Session.login("operation_registry_authority")
      #
      #   node = UU::OperationRegistry::Node.new(UU::OS::Security::Session.current_session)
      #   node.delete("ues:OT-BT[99923616732452117]:5819e14581e8a357881e5996[5819e14581e8a357881e5996]")
      #
      # @example Node deletion (forced)
      #   require "uu_operation_registry"
      #
      #   UU::OS::Security::Session.login("operation_registry_authority")
      #
      #   node = UU::OperationRegistry::Node.new(UU::OS::Security::Session.current_session)
      #   node.delete("ues:OT-BT[99923616732452117]:5819e14581e8a357881e5996[5819e14581e8a357881e5996]", force: true)
      #
      # @param node_uri [String, UU::OS::UESURI] Node URI.
      # @param parameters [Hash] Node deletion parameters.
      # @option parameters [TrueClass, FalseClass] force Flag determining
      #   whether node deletion is forced. May be +nil+ (+false+ is assumed
      #   if the parameter is not specified).
      #
      #   Use with caution! - +force: true+ deletes the node even if a proper
      #   capacity log record cannot be created (to terminate billing of the
      #   node) and/or free capacity of the related resource pool and/or
      #   host cannot be updated.
      # @return [void]
      def delete(node_uri, parameters = {})
        client.invoke(:delete, node_uri, parameters: parameters)
      end

      # Deletes the specified node from the registry. This method is a
      # convenience shortcut for the {#delete} instance method.
      # @param node_uri [String, UU::OS::UESURI] Node URI.
      # @return [void]
      def self.delete(node_uri, parameters = {})
        self.new(UU::OS::Security::Session.current_session).delete(node_uri, parameters)
      end

      # Returns a list of nodes on the given host. Command does
      # not return all nodes, only a sublist. The list of returned nodes
      # could be filtered using {https://docs.mongodb.org/manual/tutorial/query-documents/
      # Mongo Query}.
      #
      # @param host_uri [String, UU::OS::UESURI] Host URI.
      # @param criteria [Hash] Criteria for listing nodes.
      # @option criteria [String] query Mongo Query to filter the result set.
      #   See {https://docs.mongodb.org/manual/tutorial/query-documents/ Mongo Query tutorial}.
      #   Allowed query filter document fields: code, nodeSizeUri, hostUri, resourcePoolUri
      # @option criteria [Fixnum] pageIndex Result set page index.
      # @option criteria [Fixnum] pageSize Result set page size.
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def get_node_list(host_uri, criteria = {})
        @client.invoke(:getNodeList, host_uri, parameters: criteria)
      end

      # Returns list of nodes. This method is a
      # convenience shortcut for the {#get_node_list} instance method.
      #
      # @see #get_node_list
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def self.get_node_list(host_uri, criteria = {})
        self.new(UU::OS::Security::Session.current_session).get_node_list(host_uri, criteria)
      end

    end

  end
end
