require "uu/operation_registry"
require "uu/os/cmd/command_client"
require "uu/os/security/session"

module UU
  module OperationRegistry

    # Node image.
    class NodeImage < BaseMethods

      # Remote service path.
      # @private
      PATH = "uu-operation-registry/NodeImage"

      # Creates a new instance of +NodeImage+.
      #
      # @param session [UU::OS::Security::Session] Session to use for remote calls.
      def initialize(session = nil)
        super
      end

      # @!method create(cloud_uri, attributes)
      #   Creates a new node image.
      #   @param cloud_uri [String, UU::OS::UESURI] Cloud URI.
      #   @param attributes [Hash] Node image attributes.
      #   @option attributes [String] name Node image name.
      #   @option attributes [String] code Node image code. May be +nil+.
      #   @option attributes [String] description Node image description.
      #     May be +nil+.
      #   @option attributes [UU::OS::UESURI] nodeSizeUri Base class URI. May be +nil+.
      #   @option attributes [Array<Fixnum>] inboundPorts Inbound ports. May
      #     be +nil+.
      #   @option attributes [String] imageUrl Physical location
      #     of the actual node image. May be +nil+.
      #   @option attributes [UU:OS::UESURI] runtimeStackUri Runtime stack URI. May be +nil+.
      #   @option attributes [UU::OS::UESURI] appDeploymentUri UESURI of
      #     deployment that this node image belongs to. May be +nil+.
      #   @return [UU::OS::UESURI] UESURI of the created node image.

      # @!method self.create(cloud_uri, attributes)
      #   Creates a new node image. This method is a convenience shortcut for
      #   the {#create} instance method.
      #   @param cloud_uri [String, UU::OS::UESURI] Cloud URI.
      #   @param attributes [Hash] Node image attributes.
      #   @return [UU::OS::UESURI] UESURI of the created node image.

      # @!method get_attributes(node_image_uri)
      #   Gets the attributes of the node image specified by node image URI
      #   and returns its detail.
      #   @param node_image_uri [String, UU::OS::UESURI] Node image URI.
      #   @return [Hash] Node image attributes.

      # @!method self.get_attributes(node_image_uri)
      #   Gets the attributes of the node image specified by node image URI
      #   and returns its detail. This method is a convenience shortcut for
      #   the {#get_attributes} instance method.
      #   @param node_image_uri [String, UU::OS::UESURI] Node image URI.
      #   @return [Hash] Node image attributes.

      # @!method set_attributes(node_image_uri, attributes)
      #   Sets attributes of a node image.
      #   @param node_image_uri [String, UU::OS::UESURI] Node image URI.
      #   @param attributes [Hash] Node image attributes.
      #   @option attributes [String] name Node image name. May be +nil+.
      #   @option attributes [String] code Node image code. May be +nil+.
      #   @option attributes [String] description Node image description.
      #     May be +nil+.
      #   @option attributes [UU::OS::UESURI] appDeploymentUri UESURI of
      #     deployment that this node image belongs to. May be +nil+.
      #   @return [UU::OS::UESURI] UESURI of the updated node image.

      # @!method self.set_attributes(node_image_uri, attributes)
      #   Sets attributes of a node image. This method is a convenience
      #   shortcut for the {#set_attributes} instance method.
      #   @param node_image_uri [String, UU::OS::UESURI] Node image URI.
      #   @return [UU::OS::UESURI] UESURI of the updated node image.

      # @!method delete(node_image_uri)
      #   Deletes the specified node image from the registry.
      #   @param node_image_uri [String, UU::OS::UESURI] Node image URI.
      #   @return [void]

      # @!method self.delete(node_image_uri)
      #   Deletes the specified node image from the registry. This method is a
      #   convenience shortcut for the #{delete} instance method.
      #   @param node_image_uri [String, UU::OS::UESURI] Node image URI.
      #   @return [void]

      # Returns a list of node images in the given cloud. Command does
      # not return all node images, only a sublist. The list of returned node images
      # could be filtered using {https://docs.mongodb.org/manual/tutorial/query-documents/
      # Mongo Query}.
      #
      # @param cloud_uri [String, UU::OS::UESURI] Cloud URI.
      # @param criteria [Hash] Criteria for listing node images.
      # @option criteria [String] query Mongo Query to filter the result set.
      #   See {https://docs.mongodb.org/manual/tutorial/query-documents/ Mongo Query tutorial}.
      #   Allowed query filter document fields: name, code, cloudUri
      # @option criteria [Fixnum] pageIndex Result set page index.
      # @option criteria [Fixnum] pageSize Result set page size.
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def get_node_image_list(cloud_uri, criteria = {})
        @client.invoke(:getNodeImageList, cloud_uri, parameters: criteria)
      end

      # Returns list of node images. This method is a
      # convenience shortcut for the {#get_node_image_list} instance method.
      #
      # @see #get_node_image_list
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def self.get_node_image_list(cloud_uri, criteria = {})
        self.new(UU::OS::Security::Session.current_session).get_node_image_list(cloud_uri, criteria)
      end

    end

  end
end
