require "uu/operation_registry"
require "uu/os/cmd/command_client"
require "uu/os/security/session"

module UU
  module OperationRegistry

    # Node set.
    class NodeSet < BaseMethods

      # Remote service path.
      # @private
      PATH = "uu-operation-registry/NodeSet"

      # Creates a new instance of +NodeSet+.
      #
      # @param session [UU::OS::Security::Session] Session to use for remote calls.
      def initialize(session = nil)
        super
      end

      # @!method create(resource_pool_uri, attributes)
      #   Creates a new NodeSet.
      #   @param resource_pool_uri [String, UU::OS::UESURI] Resource pool URI.
      #   @param attributes [Hash] Node node set attributes.
      #   @option attributes [String] name Node set name.
      #   @option attributes [String] code Node set code. May be +nil+.
      #   @option attributes [String] urlPath URL path.
      #   @option attributes [String] balancingStrategy Balancing strategy.
      #     May be +nil+.
      #   @option attributes [UU::OS::UESURI] appDeploymentUri UESURI of
      #     deployment that this node set belongs to.
      #   @option attributes [UU::OS::UESURI] nodeSizeUri UESURI of NodeSize used by this NodeSet. May be +nil+.
      #   @option attributes [UU::OS::UESURI] nodeImageUri UESURI of NodeImage used by this NodeSet. May be +nil+.
      #   @option attributes [Array<Fixnum>] inboundPorts Inbound ports used for Swarm-NodeProvider NodeSets. May be +nil+.
      #   @option attributes [String] slot Slot to which the created NodeSets will belong to.
      #     One of +PRODUCTION+, +BETA+ or +NONE+.
      #     +NONE+ is special value which means that the NodeSet is not routed from any gateway.
      #     This value should be used only internally by C3.
      #     May be +nil+.
      #     If not specified +PRODUCTION+ slot will be used.
      #   @option attributes [String] networkCidr Network CIDR (e.g. +10.0.2.0/24+) of internal network that enables
      #     communication among nodes belonging to this node set. May be +nil+.
      #   @return [UU::OS::UESURI] UESURI of the created node set.

      # @!method self.create(resource_pool_uri, attributes)
      #   Creates a new node set. This method is a convenience shortcut for
      #   the {#create} instance method.
      #   @param resource_pool_uri [String, UU::OS::UESURI] Resource pool URI.
      #   @param attributes [Hash] Node set attributes.
      #   @return [UU::OS::UESURI] UESURI of the created node set.

      # @!method get_attributes(node_set_uri)
      #   Gets the attributes of the node set specified by container URI
      #   and returns its detail.
      #
      #   @note Attributes "nodeSizeUri", "nodeCount", "inboundPorts" and "networkCidr" are set only for ResourceGroup
      #     of type "SWARM".
      #   @param node_set_uri [String, UU::OS::UESURI] Node set URI.
      #   @return [Hash] Node set attributes.

      # @!method self.get_attributes(node_set_uri)
      #   Gets the attributes of the node set specified by container URI
      #   and returns its detail. This method is a convenience shortcut for
      #   the {#get_attributes} instance method.
      #   @param node_set_uri [String, UU::OS::UESURI] Node set URI.
      #   @return [Hash] Node set attributes.

      # @!method set_attributes(node_set_uri, attributes)
      #   Sets attributes of a node set.
      #   @param node_set_uri [String, UU::OS::UESURI] Node set URI.
      #   @param attributes [Hash] Node set attributes.
      #   @option attributes [String] name Node set name. May be +nil+.
      #   @option attributes [String] code Node set code. May be +nil+.
      #   @option attributes [String] urlPath URL path. May be +nil+.
      #   @option attributes [String] balancingStrategy Balancing strategy.
      #     May be +nil+.
      #   @option attributes [UU::OS::UESURI] nodeImageUri. URI of nodeImage.
      #   @option attributes [UU::OS::UESURI] nodeSizeUri URI of nodeSize.
      #   @option attributes [UU::OS::UESURI] appDeploymentUri UESURI of
      #     deployment that this node set belongs to. May be +nil+.
      #   @option attributes [String] slot Slot to which the created NodeSets will belong to.
      #     One of +PRODUCTION+, +BETA+ or +NONE+.
      #     +NONE+ is special value which means that the NodeSet is not routed from any gateway.
      #     This value should be used only internally by C3.
      #     May be +nil+.
      #     If not specified +PRODUCTION+ slot will be used.
      #   @option attributes [String] networkCidr Network CIDR (e.g. +10.0.2.0/24+) of internal network that enables
      #     communication among nodes belonging to this node set. May be +nil+.
      #   @return [UU::OS::UESURI] UESURI of the updated node set.

      # @!method self.set_attributes(node_set_uri, attributes)
      #   Sets attributes of a node set. This method is a convenience
      #   shortcut for the {#set_attributes} instance method.
      #   @param node_set_uri [String, UU::OS::UESURI] Node set URI.
      #   @return [UU::OS::UESURI] UESURI of the updated node set.

      # @!method delete(node_set_uri)
      #   Deletes the specified node set from the registry.
      #   @param node_set_uri [String, UU::OS::UESURI] Node set URI.
      #   @return [void]

      # @!method self.delete(node_set_uri)
      #   Deletes the specified node set from the registry. This method is a
      #   convenience shortcut for the #{delete} instance method.
      #   @param node_set_uri [String, UU::OS::UESURI] Node set URI.
      #   @return [void]

      # Returns a list of node sets in the given resource pool. Command does
      # not return all node sets, only a sublist. The list of returned node sets
      # could be filtered using {https://docs.mongodb.org/manual/tutorial/query-documents/
      # Mongo Query}.
      #
      # @note Attributes "nodeSizeUri", "nodeCount", "inboundPorts" and "networkCidr" are set only for ResourceGroup
      #   of type "SWARM".
      # @param resource_pool_uri [String, UU::OS::UESURI] Resource pool URI.
      # @param criteria [Hash] Criteria for listing node sets.
      # @option criteria [String] query Mongo Query to filter the result set.
      #   See {https://docs.mongodb.org/manual/tutorial/query-documents/ Mongo Query tutorial}.
      #   Allowed query filter document fields: code, urlPath, resourcePoolUri, networkCidr
      # @option criteria [Fixnum] pageIndex Result set page index.
      # @option criteria [Fixnum] pageSize Result set page size.
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def get_node_set_list(resource_pool_uri, criteria = {})
        @client.invoke(:getNodeSetList, resource_pool_uri, parameters: criteria)
      end

      # Returns list of node sets. This method is a
      # convenience shortcut for the {#get_node_set_list} instance method.
      #
      # @see #get_node_set_list
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def self.get_node_set_list(resource_pool_uri, criteria = {})
        self.new(UU::OS::Security::Session.current_session).get_node_set_list(resource_pool_uri, criteria)
      end

    end

  end
end
