require "uu/operation_registry"
require "uu/os/cmd/command_client"
require "uu/os/security/session"

module UU
  module OperationRegistry

    # Node Size.
    class NodeSize < BaseMethods

      # Remote service path.
      # @private
      PATH = "uu-operation-registry/NodeSize"

      # Creates a new instance of +NodeSize+.
      #
      # @param session [UU::OS::Security::Session] Session to use for remote calls.
      def initialize(session = nil)
        super
      end

      # @!method create(cloud_uri, attributes)
      #   Creates a new node size.
      #   @param cloud_uri [String, UU::OS::UESURI] Cloud URI.
      #   @param attributes [Hash] Node size attributes.
      #   @option attributes [String] name Node size name. May be +nil+.
      #   @option attributes [String] code Node size code. May be +nil+.
      #   @option attributes [String] description Node size description.
      #     May be +nil+.
      #   @option attributes [Fixnum] mem Memory limit in MB (Megabytes).
      #   @option attributes [Fixnum] cpu Number of virtual CPU cores.
      #   @option attributes [Fixnum] storage Maximum storage limit in MB (Megabytes).
      #   @return [UU::OS::UESURI] UESURI of the created node size.


      # @!method self.create(cloud_uri, attributes)
      #   Creates a new node size. This method is a convenience shortcut for
      #   the {#create} instance method.
      #   @param cloud_uri [String, UU::OS::UESURI] Cloud URI.
      #   @param attributes [Hash] Node size attributes.
      #   @return [UU::OS::UESURI] UESURI of the created node size.

      # @!method get_attributes(node_size_uri)
      #   Gets the attributes of the node size specified by node size URI
      #   and returns its detail.
      #   @param node_size_uri [String, UU::OS::UESURI] Node size URI.
      #   @return [Hash] Node size attributes.

      # @!method self.get_attributes(node_size_uri)
      #   Gets the attributes of the node size specified by node size URI
      #   and returns its detail. This method is a convenience shortcut for
      #   the {#get_attributes} instance method.
      #   @param node_size_uri [String, UU::OS::UESURI] Node size URI.
      #   @return [Hash] Node size attributes.

      # @!method set_attributes(node_size_uri, attributes)
      #   Sets attributes of a node size.
      #   @param node_size_uri [String, UU::OS::UESURI] Node size URI.
      #   @param attributes [Hash] Node size attributes.
      #   @option attributes [String] name Node size name. May be +nil+.
      #   @option attributes [String] code Node size code. May be +nil+.
      #   @option attributes [String] description Node size description.
      #     May be +nil+.
      #   @return [UU::OS::UESURI] UESURI of the updated node size.

      # @!method self.set_attributes(node_size_uri, attributes)
      #   Sets attributes of a node size. This method is a convenience
      #   shortcut for the {#set_attributes} instance method.
      #   @param node_size_uri [String, UU::OS::UESURI] Node size URI.
      #   @return [UU::OS::UESURI] UESURI of the updated node size.

      # @!method delete(node_size_uri)
      #   Deletes the specified node size from the registry.
      #   @param node_size_uri [String, UU::OS::UESURI] Node size URI.
      #   @return [void]

      # @!method self.delete(node_size_uri)
      #   Deletes the specified node size from the registry. This method is a
      #   convenience shortcut for the #{delete} instance method.
      #   @param node_size_uri [String, UU::OS::UESURI] Node size URI.
      #   @return [void]

      # Returns a list of node sizes in the given cloud. Command does
      # not return all sizes, only a sublist. The list of returned sizes
      # could be filtered using {https://docs.mongodb.org/manual/tutorial/query-documents/
      # Mongo Query}.
      #
      # @param cloud_uri [String, UU::OS::UESURI] Cloud URI.
      # @param criteria [Hash] Criteria for listing sizes.
      # @option criteria [String] query Mongo Query to filter the result set.
      #   See {https://docs.mongodb.org/manual/tutorial/query-documents/ Mongo Query tutorial}.
      #   Allowed query filter document fields: code, mem, cpu, storage, cloudUri
      # @option criteria [Fixnum] pageIndex Result set page index.
      # @option criteria [Fixnum] pageSize Result set page size.
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def get_node_size_list(cloud_uri, criteria = {})
        client.invoke(:getNodeSizeList, cloud_uri, parameters: criteria)
      end

      # Returns list of sizes. This method is a
      # convenience shortcut for the {#get_node_size_list} instance method.
      #
      # @see #get_node_size_list
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def self.get_node_size_list(cloud_uri, criteria = {})
        self.new(UU::OS::Security::Session.current_session).get_node_size_list(cloud_uri, criteria)
      end

    end

  end
end
