require "uu/operation_registry"
require "uu/os/cmd/command_client"
require "uu/os/security/session"

module UU
  module OperationRegistry

    # Resource group.
    class ResourceGroup < BaseMethods

      # Types of supported NodeProviderTypes
      class NodeProviderType
        # Swarm type.
        SWARM = 'SWARM'
        # Docker type.
        DOCKER = 'DOCKER'
      end

      # Remote service path.
      # @private
      PATH = "uu-operation-registry/ResourceGroup"

      # Creates a new instance of +ResourceGroup+.
      #
      # @param session [UU::OS::Security::Session] Session to use for remote calls.
      def initialize(session = nil)
        super
      end

      # @!method create(region_uri, attributes)
      #   Creates a new resource group.
      #   @param region_uri [String, UU::OS::UESURI] Region URI.
      #   @param attributes [Hash] ResourceGroup attributes.
      #   @option attributes [String] name ResourceGroup name. May be +nil+.
      #   @option attributes [String] code ResourceGroup code. May be +nil+.
      #   @option attributes [UU::OperationRegistry::ResourceGroup::NodeProviderType] nodeProvider Type of NodeProvider.
      #   @option attributes [Hash] nodeProviderParameters Parameters for NodeProvider.
      #     * +url+ (String) Url to Docker Swarm.
      #     * +connectionParameters+ (Hash) Additional connection parameters compatible with +Excon+ gem.
      #   @option attributes [String] description ResourceGroup description.
      #     May be +nil+.
      #   @option attributes [String] env Env configuration JSON. Contains configuration (gemrc, uu-client.properties)
      #     for nodes deployed in the context of the ResourceGroup.  May be +nil+.
      #   @option attributes [Hash] labels Hash of labels to attach to a ResourceGroup. May be +nil+.
      #   @option attributes [Hash] slotMapping Hash where keys are +slot+ names.
      #     Values of those two keys must be again Hashes containing mapping of label keys to values.
      #     May be +nil+.
      #   @option attributes [String] state ResourceGroup state. Value must be
      #     one from {UU::OperationRegistry::ResourceGroupState}. May be +nil+ in which case it will set
      #     the ResourceGroup into +ACTIVE+ state.
      #   @return [UU::OS::UESURI] UESURI of the created resource group.

      # @!method self.create(region_uri, attributes)
      #   Creates a new resource group. This method is a convenience shortcut for
      #   the {#create} instance method.
      #   @param region_uri [String, UU::OS::UESURI] Region URI.
      #   @param attributes [Hash] ResourceGroup attributes.
      #   @return [UU::OS::UESURI] UESURI of the created resource group.

      # @!method get_attributes(resource_group_uri)
      #   Gets the attributes of the resource group specified by resource group URI
      #   and returns its detail.
      #   @param resource_group_uri [String, UU::OS::UESURI] ResourceGroup URI.
      #   @return [Hash] ResourceGroup attributes.

      # @!method self.get_attributes(resource_group_uri)
      #   Gets the attributes of the resource group specified by resource group URI
      #   and returns its detail. This method is a convenience shortcut for
      #   the {#get_attributes} instance method.
      #   @param resource_group_uri [String, UU::OS::UESURI] ResourceGroup URI.
      #   @return [Hash] ResourceGroup attributes.

      # @!method set_attributes(resource_group_uri, attributes)
      #   Sets attributes of a resource group.
      #
      #   @example Set env JSON
      #    require 'uu_operation_registry'
      #    require 'json'
      #
      #    UU::OS::Security::Session.login('...')
      #
      #    env = {
      #      "uuClientProperties" => {
      #        "uu.server-url" => "https://api.plus4u.net",
      #        "uu.c3.base_url" => "https://sys.plus4u.net",
      #        "uu.c3.append_routing_data" => "false",
      #        "uu.operation.registry.base_url" => "https://sys.plus4u.net",
      #        "uu.operation.registry.append_routing_data" => "false"
      #      },
      #      "gemrc" => {
      #        ":backtrace" => "false",
      #        ":benchmark" => "false",
      #        ":bulk_threshold" => "100",
      #        ":sources" => [
      #          "http://rubygems.org",
      #          "http://gems.plus4u.net"
      #        ],
      #        ":update_sources" => "true",
      #        ":verbose" => "false",
      #        "gem" => "--env-shebang --no-doc --no-ri --no-document"
      #      }
      #    }
      #
      #    UU::OperationRegistry::ResourceGroup.set_attributes(
      #      'ues:RESOURCEGROUP:URI',
      #      :env => env.to_json
      #    )
      #
      #   @example Set labels and slotMapping
      #    require 'uu_operation_registry'
      #
      #    UU::OS::Security::Session.login('...')
      #
      #    labels = {
      #      :"shared-label" => "value"
      #    }
      #
      #    slot_mapping = {
      #      PRODUCTION: { :"uu.cloud.deployment-environment" => "group1" },
      #      BETA: { :"uu.cloud.deployment-environment" => "group2" }
      #    }
      #
      #    UU::OperationRegistry::ResourceGroup.set_attributes(
      #      'ues:RESOURCEGROUP:URI',
      #       labels: labels,
      #       slotMapping: slot_mapping
      #    )
      #
      #   @param resource_group_uri [String, UU::OS::UESURI] ResourceGroup URI.
      #   @param attributes [Hash] ResourceGroup attributes.
      #   @option attributes [String] name ResourceGroup name. May be +nil+.
      #   @option attributes [String] code ResourceGroup code. May be +nil+.
      #   @option attributes [String] description ResourceGroup description.
      #     May be +nil+.
      #   @option attributes [String] env Env configuration JSON. Contains configuration (gemrc, uu-client.properties)
      #     for nodes deployed in the context of the ResourceGroup.  May be +nil+.
      #   @option attributes [Hash] labels Hash of labels to attach to a ResourceGroup. May be +nil+.
      #   @option attributes [Hash] slotMapping Hash where keys are +slot+ names.
      #     Values of those two keys must be again Hashes containing mapping of label keys to values.
      #     May be +nil+.
      #   @option attributes [String] state ResourceGroup state. Value must be
      #     one from {UU::OperationRegistry::ResourceGroupState}. May be +nil+.
      #   @return [UU::OS::UESURI] UESURI of the updated resource group.

      # @!method self.set_attributes(resource_group_uri, attributes)
      #   Sets attributes of a resource group. This method is a convenience
      #   shortcut for the {#set_attributes} instance method.
      #   @param resource_group_uri [String, UU::OS::UESURI] ResourceGroup URI.
      #   @return [UU::OS::UESURI] UESURI of the updated resource group.

      # @!method delete(resource_group_uri)
      #   Deletes the specified resource group from the registry.
      #   @param resource_group_uri [String, UU::OS::UESURI] ResourceGroup URI.
      #   @return [void]

      # @!method self.delete(resource_group_uri)
      #   Deletes the specified resource group from the registry. This method is a
      #   convenience shortcut for the #{delete} instance method.
      #   @param resource_group_uri [String, UU::OS::UESURI] ResourceGroup URI.
      #   @return [void]

      # Returns a list of resource groups in the given region. Command does
      # not return all resource groups, only a sublist. The list of returned resource groups
      # could be filtered using {https://docs.mongodb.org/manual/tutorial/query-documents/
      # Mongo Query}.
      #
      # @param region_uri [String, UU::OS::UESURI] Region URI.
      # @param criteria [Hash] Criteria for listing resource groups.
      # @option criteria [String] query Mongo Query to filter the result set.
      #   See {https://docs.mongodb.org/manual/tutorial/query-documents/ Mongo Query tutorial}.
      # @option criteria [Fixnum] pageIndex Result set page index.
      # @option criteria [Fixnum] pageSize Result set page size.
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def get_resource_group_list(region_uri, criteria = {})
        client.invoke(:getResourceGroupList, region_uri, parameters: criteria)
      end

      # Returns list of resource groups. This method is a
      # convenience shortcut for the {#get_resource_group_list} instance method.
      #
      # @see #get_resource_group_list
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def self.get_resource_group_list(region_uri, criteria = {})
        self.new(UU::OS::Security::Session.current_session).get_resource_group_list(region_uri, criteria)
      end

    end

  end
end
