require "uu/operation_registry"
require "uu/os/cmd/command_client"
require "uu/os/security/session"

module UU
  module OperationRegistry

    # Resource lease.
    class ResourceLease < BaseMethods

      # Remote service path.
      # @private
      PATH = "uu-operation-registry/ResourceLease"

      # Creates a new instance of +Resource lease+.
      #
      # @param session [UU::OS::Security::Session] Session to use for remote calls.
      def initialize(session = nil)
        super
      end

      # @!method create(cloud_uri, attributes)
      #   Creates a new resource lease.
      #
      #   @example Create ResourceLease type MSG_BUS
      #     require "uu_operation_registry"
      #
      #     UU::OS::Security::Session.login("or-authority")
      #
      #     uucloud_uri = "ues:OT-BT:PLUS4U_CLOUD"
      #
      #     # Lease 10 queuePairs from the group of message busses with the following
      #     # labels: ["DEV"] (only message busses with the same labels as specified
      #     # in constraints are included -- i.e. a message bus with labels ["DEV", "DEV116"]
      #     # is not included)
      #     UU::OperationRegistry::ResourceLease.create( 
      #       uucloud_uri,
      #       resourceType: 'MSG_BUS',
      #       resourcePoolUri: 'ues:DEV1234-BT:DEV',
      #       constraints: {labels: ["DEV"]},
      #       contractedCapacity: {queuePairs: 10}
      #     )
      #
      #
      #   @example Create ResourceLease type DATA_STORE
      #     require "uu_operation_registry"
      #
      #     UU::OS::Security::Session.login("or-authority")
      #
      #     uucloud_uri = "ues:OT-BT:PLUS4U_CLOUD"
      #
      #     UU::OperationRegistry::ResourceLease.create(
      #       uucloud_uri,
      #       resourceType: 'DATA_STORE',
      #       constraints: {vendor: "UU"},
      #       resourceUri: 'ues:OT-BT[99923616732452117]:SECRET[59d21aadf8458a30561dc888]'
      #     )
      #
      #   @param cloud_uri [String, UU::OS::UESURI] Cloud URI.
      #   @param attributes [Hash] Resource lease attributes.
      #   @option attributes [String] name Name of the resource lease. May be +nil+.
      #   @option attributes [String] code Code of the resource lease. May be +nil+.
      #   @option attributes [String] resourceType Type of leased resource.
      #     Supported resourceTypes: MSG_BUS, DATA_STORE.
      #   @option attributes [UU::OS::UESURI] resourcePoolUri URI of resource pool 
      #     owning the lease. May be +nil+.
      #   @option attributes [Hash] constraints Resource lease constraints. May be +nil+.
      #     * Supported constraints attributes for resource type MSG_BUS: labels: (Array<String>)
      #     * Supported constraints attributes for resource type DATA_STORE: vendor: (String)
      #   @option attributes [Hash] contractedCapacity Contracted capacity of the resource lease. May be +nil+.
      #     *  Supported contractedCapacity attributes for resource type MSG_BUS: queuePairs: (Fixnum)
      #     *  Supported contractedCapacity attributes for resource type DATA_STORE: +nil+.
      #   @return [UU::OS::UESURI] UESURI of the created resource lease.

      # @!method self.create(cloud_uri, attributes)
      #   Creates a new resource lease. This method is a convenience shortcut for
      #   the {#create} instance method.
      #   @param cloud_uri [String, UU::OS::UESURI] Cloud URI.
      #   @param attributes [Hash] Resource lease attributes.
      #   @return [UU::OS::UESURI] UESURI of the created resource lease.

      # @!method get_attributes(resource_lease_uri)
      #   Gets the attributes of the resource lease specified by the URI.
      #   @param resource_lease_uri [String, UU::OS::UESURI] Resource lease URI.
      #   @return [Hash] Resource lease attributes.

      # @!method self.get_attributes(resource_lease_uri)
      #   Gets the attributes of the resource lease specified by the URI.
      #   This method is a convenience shortcut for the {#get_attributes}
      #   instance method.
      #   @param resource_lease_uri [String, UU::OS::UESURI] Resource lease URI.
      #   @return [Hash] Resource lease attributes.

      # @!method set_attributes(resource_lease_uri, attributes)
      #   Sets attributes of a resource lease.
      #   @param resource_lease_uri [String, UU::OS::UESURI] Resource lease URI.
      #   @param attributes [Hash] Resource lease attributes.
      #   @option attributes [String] name Name of the resource lease. May be +nil+.
      #   @option attributes [String] code Code of the resource lease. May be +nil+.
      #   @option attributes [UU::OS::UESURI] resourcePoolUri URI of resource pool 
      #     owning the lease. May be +nil+.
      #   @option attributes [Hash] constraints Resource lease constraints. May be +nil+.
      #     Supported constraints attributes for resource type MSG_BUS: labels: (Array<String>)
      #     Supported constraints attributes for resource type DATA_STORE: vendor: (String)
      #   @option attributes [Hash] contractedCapacity Contracted capacity of the resource lease. May be +nil+.
      #     *  Supported contractedCapacity attributes for resource type MSG_BUS: queuePairs: (Fixnum)
      #     *  Supported contractedCapacity attributes for resource type DATA_STORE: +nil+.
      #   @option attributes [Hash] freeCapacity Free capacity of the reasource lease. May be +nil+
      #     Hash format is identical to the format of contractedCapacity Hash.
      #   @return [UU::OS::UESURI] UESURI of the updated resource lease.

      # @!method self.set_attributes(resource_lease_uri, attributes)
      #   Sets attributes of a resource lease. This method is a convenience
      #   shortcut for the {#set_attributes} instance method.
      #   @param resource_lease_uri [String, UU::OS::UESURI] Resource lease URI.
      #   @return [UU::OS::UESURI] UESURI of the updated resource lease.

      # @!method delete(resource_lease_uri)
      #   Deletes the specified resource lease from the registry.
      #   @param resource_lease_uri [String, UU::OS::UESURI] Resource lease URI.
      #   @return [void]

      # @!method self.delete(resource_lease_uri)
      #   Deletes the specified resource lease from the registry. This method is a
      #   convenience shortcut for the {#delete} instance method.
      #   @param resource_lease_uri [String, UU::OS::UESURI] Resource lease URI.
      #   @return [void]

      # Returns a list of resource leases in the given cloud. Command does
      # not return all resource leases, only a sublist. The list of returned resource leases
      # could be filtered using {https://docs.mongodb.org/manual/tutorial/query-documents/
      # Mongo Query}.
      #
      # @param cloud_uri [String, UU::OS::UESURI] Cloud URI.
      # @param criteria [Hash] Criteria for listing leases.
      # @option criteria [String] query Mongo Query to filter the result set.
      #   See {https://docs.mongodb.org/manual/tutorial/query-documents/ Mongo Query tutorial}.
      #   Allowed query filter document fields:
      #     code, resourceType, resourcePoolUri, constraints, contractedCapacity, freeCapacity
      # @option criteria [Fixnum] pageIndex Result set page index.
      # @option criteria [Fixnum] pageSize Result set page size.
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def get_resource_lease_list(cloud_uri, criteria = {})
        @client.invoke(:getResourceLeaseList, cloud_uri, parameters: criteria)
      end

      # Returns list of resource leases. This method is a
      # convenience shortcut for the {#get_resource_lease_list} instance method.
      #
      # @see #get_resource_lease_list
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def self.get_resource_lease_list(cloud_uri, criteria = {})
        self.new(UU::OS::Security::Session.current_session).get_resource_lease_list(cloud_uri, criteria)
      end

    end
  end
end
