require "uu/operation_registry"
require "uu/os/cmd/command_client"
require "uu/os/security/session"

module UU
  module OperationRegistry

    # Resource pool.
    class ResourcePool < BaseMethods

      # Remote service path.
      # @private
      PATH = "uu-operation-registry/ResourcePool"

      # Creates a new instance of +Resource pool+.
      #
      # @param session [UU::OS::Security::Session] Session to use for remote calls.
      def initialize(session = nil)
        super
      end

      # @!method create(tenant_uri, attributes)
      #   Creates a new resource pool.
      #   @param tenant_uri [String, UU::OS::UESURI] Tenant URI.
      #   @param attributes [Hash] Resource pool attributes.
      #   @option attributes [String] name Resource pool name. May be +nil+.
      #   @option attributes [String] code Resource pool code. May be +nil+.
      #   @option attributes [String] description Resource pool description.
      #     May be +nil+.
      #   @option attributes [UU::OS::UESURI] accessGroupUri URI of group used
      #     to verify access rights manipulate with the resource pool (e.g. deploy
      #     apps to it).
      #   @option attributes [UU::OS::UESURI] auditorsUri URI of group used
      #      to verify auditors rights (e.g. read logs from deployed apps). May be +nil+.
      #   @option attributes [Hash] contractedCapacity Contracted capacity for
      #     this resource pool. Supported keys and value types:
      #     !{cpu: Numeric, mem: Fixnum, storage: Fixnum}. All keys must
      #     have values.
      #     * +cpu+ (Fixnum) Number of virtual CPU cores.
      #     * +mem+ (Fixnum) Memory limit in MB (Megabytes).
      #     * +storage+ (Fixnum) Maximum storage limit in MB (Megabytes).
      #   @option attributes [UU::OS::UESURI] resourceGroupUri URI of resource group
      #     that resources in this pool belong to.
      #   @option attributes [String] env Env configuration JSON. Contains configuration (gemrc, uu-client.properties)
      #     for nodes deployed in the context of the Region.  May be +nil+.
      #   @return [UU::OS::UESURI] UESURI of the created pool.

      # @!method self.create(tenant_uri, attributes)
      #   Creates a new resource pool. This method is a convenience shortcut for
      #   the {#create} instance method.
      #   @param tenant_uri [String, UU::OS::UESURI] Tenant URI.
      #   @param attributes [Hash] Resource pool attributes.
      #   @return [UU::OS::UESURI] UESURI of the created pool.

      # @!method get_attributes(resource_pool_uri)
      #   Gets the attributes of the pool specified by pool URI
      #   and returns its detail.
      #   @param resource_pool_uri [String, UU::OS::UESURI] Resource pool URI.
      #   @return [Hash] Resource pool attributes.

      # @!method self.get_attributes(resource_pool_uri)
      #   Gets the attributes of the pool specified by pool URI
      #   and returns its detail. This method is a convenience shortcut for
      #   the {#get_attributes} instance method.
      #   @param resource_pool_uri [String, UU::OS::UESURI] Resource pool URI.
      #   @return [Hash] Resource pool attributes.

      # @!method set_attributes(resource_pool_uri, attributes)
      #   Sets attributes of a pool.
      #
      #   @example Set env JSON
      #    require 'uu_operation_registry'
      #    require 'json'
      #
      #    UU::OS::Security::Session.login('...')
      #
      #    env = {
      #      "uuClientProperties" => {
      #        "uu.server-url" => "https://api.plus4u.net",
      #        "uu.c3.base_url" => "https://sys.plus4u.net",
      #        "uu.c3.append_routing_data" => "false",
      #        "uu.operation.registry.base_url" => "https://sys.plus4u.net",
      #        "uu.operation.registry.append_routing_data" => "false"
      #      },
      #      "gemrc" => {
      #        ":backtrace" => "false",
      #        ":benchmark" => "false",
      #        ":bulk_threshold" => "100",
      #        ":sources" => [
      #          "http://rubygems.org",
      #          "http://gems.plus4u.net"
      #        ],
      #        ":update_sources" => "true",
      #        ":verbose" => "false",
      #        "gem" => "--env-shebang --no-doc --no-ri --no-document"
      #      }
      #    }
      #
      #    UU::OperationRegistry::ResourcePool.set_attributes('ues:DEV1234-BT:DEV',
      #      env: env.to_json
      #    )
      #
      #   @example Set monitoring base URL
      #    UU::OperationRegistry::ResourcePool.set_attributes('ues:DEV1234-BT:DEV',
      #      monitoringBaseUrl: 'https://telemetry.plus4u.net/uu-cloudg01-monitoring/1234-59955e0d123ddbdfe046c6f8/'
      #    )
      #
      #   @param resource_pool_uri [String, UU::OS::UESURI] Resource pool URI.
      #   @param attributes [Hash] Resource pool attributes.
      #   @option attributes [String] name Resource pool name. May be +nil+.
      #   @option attributes [String] code Resource pool code. May be +nil+.
      #   @option attributes [String] description Resource pool description.
      #     May be +nil+.
      #   @option attributes [UU::OS::UESURI] accessGroupUri URI of group used
      #     to verify access rights manipulate with the resource pool (e.g. deploy
      #     apps to it).
      #   @option attributes [UU::OS::UESURI] auditorsUri URI of group used
      #      to verify auditors rights (e.g. read logs from deployed apps). May be +nil+.
      #   @option attributes [Hash] contractedCapacity Contracted capacity for
      #     this resource pool. Supported keys and value types:
      #     !{cpu: Numeric, mem: Fixnum, storage: Fixnum}. All values are
      #     optional.
      #     * +cpu+ (Fixnum) Number of virtual CPU cores.
      #     * +mem+ (Fixnum) Memory limit in MB (Megabytes).
      #     * +storage+ (Fixnum) Maximum storage limit in MB (Megabytes).
      #   @option attributes [String] env Env configuration JSON. Contains configuration (gemrc, uu-client.properties)
      #     for nodes deployed in the context of the Region.  May be +nil+.
      #   @option attributes [String] monitoringBaseUrl URL of uuCloudg01Monitoring instance, if deployed.
      #     May be +nil+.
      #   @return [UU::OS::UESURI] UESURI of the updated pool.

      # @!method self.set_attributes(resource_pool_uri, attributes)
      #   Sets attributes of a pool. This method is a convenience
      #   shortcut for the {#set_attributes} instance method.
      #   @param resource_pool_uri [String, UU::OS::UESURI] Resource pool URI.
      #   @return [UU::OS::UESURI] UESURI of the updated pool.

      # @!method delete(resource_pool_uri)
      #   Deletes the specified pool from the registry.
      #   @param resource_pool_uri [String, UU::OS::UESURI] Resource pool URI.
      #   @return [void]

      # @!method self.delete(resource_pool_uri)
      #   Deletes the specified pool from the registry. This method is a
      #   convenience shortcut for the #{delete} instance method.
      #   @param resource_pool_uri [String, UU::OS::UESURI] Resource pool URI.
      #   @return [void]

      # Returns a list of resource pools in the given tenant. Command does
      # not return all resource pools, only a sublist. The list of returned resource pools
      # could be filtered using {https://docs.mongodb.org/manual/tutorial/query-documents/
      # Mongo Query}.
      #
      # @param tenant_uri [String, UU::OS::UESURI] Tenant URI.
      # @param criteria [Hash] Criteria for listing pools.
      # @option criteria [String] query Mongo Query to filter the result set.
      #   See {https://docs.mongodb.org/manual/tutorial/query-documents/ Mongo Query tutorial}.
      #   Allowed query filter document fields:
      #     code, tenantUri, contractedCapacity, freeCapacity, accessGroupUri, resourceGroupUri
      # @option criteria [Fixnum] pageIndex Result set page index.
      # @option criteria [Fixnum] pageSize Result set page size.
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def get_resource_pool_list(tenant_uri, criteria = {})
        @client.invoke(:getResourcePoolList, tenant_uri, parameters: criteria)
      end

      # Returns list of resource pools. This method is a
      # convenience shortcut for the {#get_resource_pool_list} instance method.
      #
      # @see #get_resource_pool_list
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def self.get_resource_pool_list(tenant_uri, criteria = {})
        self.new(UU::OS::Security::Session.current_session).get_resource_pool_list(tenant_uri, criteria)
      end

      # Allocates new capacity in ResourcePool and ResourceGroup.
      # Also creates a new record to CapacityLog.
      # Usable for ResourceGroup and ResourcePool managed by SwarmNodeProvider.
      #
      # @param resource_pool_uri [String, UU::OS::UESURI] ResourcePool URI.
      # @param criteria [Hash] Criteria for allocating capacity.
      # @option criteria [String] nodeSetUri NodeSet URI.
      # @option criteria [String] nodeCount Number of a new allocated nodes.
      # @return [NilClass]
      def allocate_capacity(resource_pool_uri, criteria = {})
        @client.invoke(:allocateCapacity, resource_pool_uri, parameters: criteria)
      end

      # Allocates new capacity in ResourcePool and ResourceGroup. This method is a
      # convenience shortcut for the {#allocate_capacity} instance method.
      #
      # @see #allocate_capacity
      # @return [NilClass]
      def self.allocate_capacity(resource_pool_uri, criteria = {})
        self.new(UU::OS::Security::Session.current_session).allocate_capacity(resource_pool_uri, criteria)
      end

      # Releases capacity allocated in ResourcePool and ResourceGroup.
      # Also creates a new record to CapacityLog.
      # Usable for ResourceGroup and ResourcePool managed by SwarmNodeProvider.
      #
      # @param resource_pool_uri [String, UU::OS::UESURI] ResourcePool URI.
      # @param criteria [Hash] Criteria for allocating capacity.
      # @option criteria [String] nodeSetUri NodeSet URI.
      # @option criteria [String] nodeCount Number of a released nodes.
      # @return [NilClass]
      def deallocate_capacity(resource_pool_uri, criteria = {})
        @client.invoke(:deallocateCapacity, resource_pool_uri, parameters: criteria)
      end

      # Releases capacity allocated in ResourcePool and ResourceGroup. This method is a
      # convenience shortcut for the {#deallocate_capacity} instance method.
      #
      # @see #deallocate_capacity
      # @return [NilClass]
      def self.deallocate_capacity(resource_pool_uri, criteria = {})
        self.new(UU::OS::Security::Session.current_session).deallocate_capacity(resource_pool_uri, criteria)
      end

    end

  end
end
