require "uu/operation_registry"
require "uu/os/cmd/command_client"
require "uu/os/security/session"

module UU
  module OperationRegistry

    # Secret.
    class Secret < BaseMethods

      # Remote service path.
      # @private
      PATH = "uu-operation-registry/Secret"

      # Creates a new instance of +Secret+.
      #
      # @param session [UU::OS::Security::Session] Session to use for remote calls.
      def initialize(session = nil)
        super
      end

      # @!method create(cloud_uri, attributes)
      #   Creates a new Secret.
      #   @param cloud_uri [String, UU::OS::UESURI] Cloud URI.
      #   @param attributes [Hash] Secret attributes.
      #   @option attributes [String] name Secret name. May be +nil+.
      #   @option attributes [String] code Serves as identifier (for example OSID/BSID)
      #     for a secret value. 
      #   @option attributes [String] value Secret value. 
      #   @return [UU::OS::UESURI] UESURI of the created Secret.

      # @!method self.create(cloud_uri, attributes)
      #   Creates a new Secret. This method is a convenience shortcut for
      #   the {#create} instance method.
      #   @param cloud_uri [String, UU::OS::UESURI] Cloud URI.
      #   @param attributes [Hash] Secret attributes.
      #   @return [UU::OS::UESURI] UESURI of the created Secret.
      #   @example Create example
      #     
      #     secret = {
      #       name: "OSID mapping",
      #       code: "v3dphou7k303n2gzfklsxzon9ymlo817",
      #       value: "mongodb://v3dphou7k303n2gzfklsxzon9ymlo817__b933l3ojvini92p2o9ongo-o-s.eu-n-1.cloud.plus4u.net:27017/v3dphou7k303n2gzfklsxzon9ymlo817"
      #     }
      #
      #     UU::OperationRegistry::Secret.create(cloud_uri, secret)

      # @!method get_attributes(secret_uri)
      #   Gets the attributes of the Secret specified by secret URI.
      #   @param secret_uri [String, UU::OS::UESURI] Secret URI.
      #   @return [Hash] Secret attributes.

      # @!method self.get_attributes(secret_uri)
      #   Gets the attributes of the Secret specified by secret URI.
      #   This method is a convenience shortcut for
      #   the {#get_attributes} instance method.
      #   @param secret_uri [String, UU::OS::UESURI] Secret URI.
      #   @return [Hash] Secret attributes.

      # @!method set_attributes(secret_uri, attributes)
      #   Sets attributes of a Secret.
      #   @param secret_uri [String, UU::OS::UESURI] Secret URI.
      #   @param attributes [Hash] Secret attributes to update.
      #   @option attributes [String] name Secret name. May be +nil+.
      #   @option attributes [String] code Serves as identifier (for example OSID/BSID)
      #     for a secret value. 
      #   @option attributes [String] value Secret value.
      #   @return [UU::OS::UESURI] UESURI of the updated Secret.

      # @!method self.set_attributes(secret_uri, attributes)
      #   Sets attributes of a Secret. This method is a convenience
      #   shortcut for the {#set_attributes} instance method.
      #   @param secret_uri [String, UU::OS::UESURI] Secret URI.
      #   @return [UU::OS::UESURI] UESURI of the updated Secret.

      # @!method delete(secret_uri)
      #   Deletes the specified Secret from the registry.
      #   @param secret_uri [String, UU::OS::UESURI] Secret URI.
      #   @return [void]

      # @!method self.delete(secret_uri)
      #   Deletes the specified Secret from the registry. This method is a
      #   convenience shortcut for the #{delete} instance method.
      #   @param secret_uri [String, UU::OS::UESURI] Secret URI.
      #   @return [void]

      # Returns a list of Secrets for the given Cloud. Command does
      # not return all Secrets, only a sublist. The list of returned Secrets
      # could be filtered using {https://docs.mongodb.org/manual/tutorial/query-documents/
      # Mongo Query}.
      #
      # @param cloud_uri [String, UU::OS::UESURI] Cloud URI.
      # @param criteria [Hash] Criteria for listing Secrets.
      # @option criteria [String] query Mongo Query to filter the result set.
      #   See {https://docs.mongodb.org/manual/tutorial/query-documents/ Mongo Query tutorial}.
      #   Allowed query filter document fields: code, value. 
      # @option criteria [Fixnum] pageIndex Result set page index.
      # @option criteria [Fixnum] pageSize Result set page size.
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def get_secret_list(cloud_uri, criteria = {})
        @client.invoke(:getSecretList, cloud_uri, parameters: criteria)
      end

      # Returns list of Secrets. This method is a
      # convenience shortcut for the {#get_secret_list} instance method.
      #
      # @see #get_secret_list
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def self.get_secret_list(cloud_uri, criteria = {})
        self.new(UU::OS::Security::Session.current_session).get_secret_list(cloud_uri, criteria)
      end

    end

  end
end
