require "uu/operation_registry"
require "uu/os/cmd/command_client"
require "uu/os/security/session"

module UU
  module OperationRegistry

    # Tenant.
    class Tenant < BaseMethods

      # Remote service path.
      # @private
      PATH = "uu-operation-registry/Tenant"

      # Creates a new instance of +Tenant+.
      #
      # @param session [UU::OS::Security::Session] Session to use for remote calls.
      def initialize(session = nil)
        super
      end

      # @!method create(cloud_uri, attributes)
      #   Creates a new tenant. Tenant is a representation of territory in operation registry.
      #   @param cloud_uri [String, UU::OS::UESURI] Cloud URI.
      #   @param attributes [Hash] Tenant attributes.
      #   @option attributes [String] oid Unique territory ID.
      #     Must not be +nil+ nor empty.
      #   @option attributes [String] name Tenant name. May be +nil+.
      #   @option attributes [String] code Tenant code.
      #   @option attributes [String] tid OID of the related tenant representing territory
      #     (with +type+: +TID+). May be +nil+ unless +type+ is specified.
      #   @option attributes [String] type Tenant type. May be +nil+.
      #     Supported tenant types are:
      #
      #     * +ASID+ - represents uuAppServer (ASID = AppServer ID), belongs to a +TID+ tenant
      #     * +AWID+ - represents uuAppWorkspace (AWID = AppWorkspace ID), belongs to a +TID+ tenant
      #     * +TID+ - represents Business Territory or My Territory (TID = Territory ID)
      #   @option attributes [String] description Tenant description.
      #     May be +nil+.
      #   @return [UU::OS::UESURI] UESURI of the created tenant.

      # @!method self.create(cloud_uri, attributes)
      #   Creates a new tenant. This method is a convenience shortcut for
      #   the {#create} instance method.
      #   @param cloud_uri [String, UU::OS::UESURI] Cloud URI.
      #   @param attributes [Hash] Tenant attributes.
      #   @return [UU::OS::UESURI] UESURI of the created tenant.

      # @!method get_attributes(tenant_uri)
      #   Gets the attributes of the tenant specified by tenant URI
      #   and returns its detail.
      #   @param tenant_uri [String, UU::OS::UESURI] Tenant URI.
      #   @return [Hash] Tenant attributes.

      # @!method self.get_attributes(tenant_uri)
      #   Gets the attributes of the tenant specified by tenant URI
      #   and returns its detail. This method is a convenience shortcut for
      #   the {#get_attributes} instance method.
      #   @param tenant_uri [String, UU::OS::UESURI] Tenant URI.
      #   @return [Hash] Tenant attributes.

      # @!method set_attributes(tenant_uri, attributes)
      #   Sets attributes of an tenant.
      #   @param tenant_uri [String, UU::OS::UESURI] Tenant URI.
      #   @param attributes [Hash] Tenant attributes.
      #   @option attributes [String] oid Unique territory ID.
      #     May be +nil+ but must not be empty when specified.
      #   @option attributes [String] name Tenant name. May be +nil+.
      #   @option attributes [String] code Tenant code. May be +nil+.
      #   @option attributes [String] tid OID of the related tenant representing territory
      #     (with +type+: +TID+). May be +nil+ unless +type+ is specified.
      #   @option attributes [String] type Tenant type. May be +nil+.
      #     Supported tenant types are: +ASID+, +AWID+, +TID+. See {#create}
      #     for details.
      #   @option attributes [String] description Tenant description.
      #     May be +nil+.
      #   @return [UU::OS::UESURI] UESURI of the updated tenant.

      # @!method self.set_attributes(tenant_uri, attributes)
      #   Sets attributes of an tenant. This method is a convenience
      #   shortcut for the {#set_attributes} instance method.
      #   @param tenant_uri [String, UU::OS::UESURI] Tenant URI.
      #   @return [UU::OS::UESURI] UESURI of the updated tenant.

      # @!method delete(tenant_uri)
      #   Deletes the specified tenant from the registry.
      #   @param tenant_uri [String, UU::OS::UESURI] Tenant URI.
      #   @return [void]

      # @!method self.delete(tenant_uri)
      #   Deletes the specified tenant from the registry. This method is a
      #   convenience shortcut for the #{delete} instance method.
      #   @param tenant_uri [String, UU::OS::UESURI] Tenant URI.
      #   @return [void]

      # Returns a list of tenants in the given cloud. Command does
      # not return all tenants, only a sublist. The list of returned tenants
      # could be filtered using {https://docs.mongodb.org/manual/tutorial/query-documents/
      # Mongo Query}.
      #
      # @param cloud_uri [String, UU::OS::UESURI] Cloud URI.
      # @param criteria [Hash] Criteria for listing tenants.
      # @option criteria [String] query Mongo Query to filter the result set.
      #   See {https://docs.mongodb.org/manual/tutorial/query-documents/ Mongo Query tutorial}.
      #   Allowed query filter document fields: oid, code, cloudUri
      # @option criteria [Fixnum] pageIndex Result set page index.
      # @option criteria [Fixnum] pageSize Result set page size.
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def get_tenant_list(cloud_uri, criteria = {})
        @client.invoke(:getTenantList, cloud_uri, parameters: criteria)
      end

      # Returns list of tenants. This method is a
      # convenience shortcut for the {#get_tenant_list} instance method.
      #
      # @see #get_tenant_list
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def self.get_tenant_list(cloud_uri, criteria = {})
        self.new(UU::OS::Security::Session.current_session).get_tenant_list(cloud_uri, criteria)
      end

    end

  end
end
