require "uu/os/cmd/command_client"
require "uu/os/security/session"
require "uu/operation_registry"

module UU
  module OperationRegistry
    # UueeWhitelist
    class UUEE

      # Remote service path.
      # @private
      PATH = "uu-operation-registry/UUEE"

      # Creates a new instance of +UueeWhitelist+.
      #
      # @param session [UU::OS::Security::Session] Session to use for remote calls.
      def initialize(session = nil)
        UU::OperationRegistry::BaseMethods._init_command_client(PATH)
        @client = UU::OS::CMD::CommandClient.new(PATH, session)
      end

      # Add new entry.
      # @param cloud_uri [String, UU::OS::UESURI] Cloud URI.
      # @param parameters [Hash] Entry attributes.
      # @option parameters [String] app vendor - app code of application.
      #   Must be set.
      # @option parameters [String] tenant Tenant code.
      #   Possible to use wildcard character.
      #   For example: If you want to use all tenant codes starts with "DEV", you can set "DEV*".
      # @option parameters [Array<String>] uuEEs List of entry uuEEs. Minimal one uuEE must be set.
      # @option parameters [String] appIdentity uuEE used as application identity.
      #   First uuEE from uuEE list will be used if appIdentity wasn't set.
      #   May be +nil+.
      # @return  [String] New entry URI.
      def add_entry(cloud_uri, parameters = {})
        @client.invoke(:addEntry, cloud_uri, :parameters => parameters)
      end

      # Add new entry.
      # @param cloud_uri [String, UU::OS::UESURI] Cloud URI.
      # @param parameters [Hash] Entry attributes.
      # @option parameters [String] app vendor - app code of application.
      #   Must be set.
      # @option parameters [String] tenant Tenant code.
      #   Possible to use wildcard character.
      #   For example: If you want to use all tenant codes starts with "DEV", you can set "DEV*".
      # @option parameters [Array<String>] uuEEs List of entry uuEEs. Minimal one uuEE must be set.
      # @option parameters [String] appIdentity uuEE used as application identity.
      #   First uuEE from uuEE list will be used if appIdentity wasn't set.
      #   May be +nil+.
      # @return  [String] New entry URI.
      def self.add_entry(cloud_uri, parameters = {})
        self.new(UU::OS::Security::Session.current_session).add_entry(cloud_uri, parameters)
      end

      # Returns hash of entries paged and filtered by criteria.
      # @param cloud_uri [String, UU::OS::UESURI] Cloud URI.
      # @param criteria [Hash] List criteria.
      # @option criteria [Fixnum] pageSize Result set page size.
      # @option criteria [Fixnum] pageIndex Result set page index.
      # @option criteria [String] query Mongo Query to filter the result set.
      #   See {https://docs.mongodb.org/manual/tutorial/query-documents/ Mongo Query tutorial}.
      #   Allowed query filter document fields: code, cloudUri, app, tenant, appIdentity
      # @return [Hash] List of entries.
      # @example Example of returned hash structure.
      #  {
      #      :pageEntries => [
      #          {
      #              :app => "TEST.APP",
      #              :tenant => "DEV",
      #              :appIdentity => "1-2",
      #              :uuEEs => [
      #                  "1-2"
      #              ],
      #              :creationTime => "2017-03-03T09:30:46Z",
      #              :creationStamp => "Josef Novák (24-36-1)",
      #              :modificationTime => "2017-03-03T09:30:46Z",
      #              :modificationStamp => "Josef Novák (24-36-1)",
      #              :uri => "ues:OT-BT[0]:PLUS4U_CLOUD[58b937c6b47a229158e9a100]:58b937c7b47a229158e9a102[58b937c7b47a229158e9a102]"
      #          }
      #      ],
      #      :criteria => {
      #          :pageIndex => 0,
      #          :pageSize => 1000
      #      },
      #      :totalSize => 1
      #  }
      def get_entry_list(cloud_uri, criteria = {})
        @client.invoke(:getEntryList, cloud_uri, :parameters => criteria)
      end


      # Returns hash of entries paged and filtered by criteria.
      # @param cloud_uri [String, UU::OS::UESURI] Cloud URI.
      # @param criteria [Hash] List criteria.
      # @option criteria [Fixnum] pageSize Result set page size.
      # @option criteria [Fixnum] pageIndex Result set page index.
      # @option criteria [String] query Mongo Query to filter the result set.
      #   See {https://docs.mongodb.org/manual/tutorial/query-documents/ Mongo Query tutorial}.
      #   Allowed query filter document fields: code, cloudUri, app, tenant, appIdentity
      # @return [Hash] List of entries.
      # @see #get_entry_list
      def self.get_entry_list(cloud_uri, criteria = {})
        self.new(UU::OS::Security::Session.current_session).get_entry_list(cloud_uri, criteria)
      end

      # Remove entry.
      # @param entry_uri [String, UU::OS::UESURI] Cloud URI.
      # @return [String] URI of removed entry.
      def remove_entry(entry_uri)
        @client.invoke(:removeEntry, entry_uri)
      end

      # Remove entry.
      # @param entry_uri [String, UU::OS::UESURI] Cloud URI.
      # @return [String] URI of removed entry.
      def self.remove_entry(entry_uri)
        self.new(UU::OS::Security::Session.current_session).remove_entry(entry_uri)
      end

    end
  end
end
