require "uu/operation_registry"
require "uu/os/cmd/command_client"
require "uu/os/security/session"

module UU
  module OperationRegistry

    # Volume.
    class Volume < BaseMethods

      # Remote service path.
      # @private
      PATH = "uu-operation-registry/Volume"

      # Creates a new instance of +Volume+.
      #
      # @param session [UU::OS::Security::Session] Session to use for remote calls.
      def initialize(session = nil)
        super
      end

      # @!method create(cloud_uri, attributes)
      #   Creates a new Volume.
      #   @param cloud_uri [String, UU::OS::UESURI] Cloud URI.
      #   @param attributes [Hash] Volume attributes.
      #   @option attributes [String] name Volume name. May be +nil+.
      #   @option attributes [String] code Serves as identifier for a volume. May be +nil+.
      #   @option attributes [String, UU::OS::UESURI] appDeploymentUri URI of AppDeployment the Volume is assigned to.
      #   @option attributes [String] driver Name of Docker volume plugin to use;
      #     the only currently supported value is +cloudstor:azure+
      #   @option attributes [String] source Identifier of the persistent storage on the cloud platform.
      #     The recommended format is +hexa32code+.
      #     E.g. name of File Share in Azure Files to use as backing storage.
      #   @option attributes [String] destination The path in the particular uuNode where the volume shall be mounted.
      #   @return [UU::OS::UESURI] UESURI of the created Volume.

      # @!method self.create(cloud_uri, attributes)
      #   Creates a new Volume. This method is a convenience shortcut for
      #   the {#create} instance method.
      #   @param cloud_uri [String, UU::OS::UESURI] Cloud URI.
      #   @param attributes [Hash] Volume attributes.
      #   @option attributes [String] name Volume name. May be +nil+.
      #   @option attributes [String] code Serves as identifier for a volume. May be +nil+.
      #   @option attributes [String, UU::OS::UESURI] appDeploymentUri URI of AppDeployment the Volume is assigned to.
      #   @option attributes [String] driver Name of Docker volume plugin to use;
      #     the only currently supported value is +cloudstor:azure+
      #   @option attributes [String] source Identifier of the persistent storage on the cloud platform.
      #     The recommended format is +hexa32code+.
      #     E.g. name of File Share in Azure Files to use as backing storage.
      #   @option attributes [String] destination The path in the particular uuNode where the volume shall be mounted.
      #   @return [UU::OS::UESURI] UESURI of the created Volume.
      #   @example Create example
      #
      #     volume = {
      #       name: "My First Volume",
      #       driver: "cloudstor:azure",
      #       source: "80fb27714e674b3b90389b92da1fe947",
      #       destination: "/data",
      #       appDeploymentUri: UU::OS::UESURI.new("ues:[78462435]:[571fb9da2792891f35e22919]:[5739c55927928929b914820f]"),
      #       code: "a357881e59965819e14581e8",
      #     }
      #
      #     UU::OperationRegistry::Volume.create(cloud_uri, volume)

      # @!method get_attributes(volume_uri)
      #   Gets the attributes of the Volume specified by volume URI.
      #   @param volume_uri [String, UU::OS::UESURI] Volume URI.
      #   @return [Hash] Volume attributes.

      # @!method self.get_attributes(volume_uri)
      #   Gets the attributes of the Volume specified by volume URI.
      #   This method is a convenience shortcut for
      #   the {#get_attributes} instance method.
      #   @param volume_uri [String, UU::OS::UESURI] Volume URI.
      #   @return [Hash] Volume attributes.

      # @!method set_attributes(volume_uri, attributes)
      #   Sets attributes of a Volume.
      #   @param volume_uri [String, UU::OS::UESURI] Volume URI.
      #   @param attributes [Hash] Volume attributes to update.
      #   @option attributes [String] name Volume name. May be +nil+.
      #   @option attributes [String] code Serves as identifier for a volume.
      #   @option attributes [String] source Identifier of the persistent storage on the cloud platform.
      #     The recommended format is +hexa32code+.
      #     E.g. name of File Share in Azure Files to use as backing storage.
      #   @option attributes [String] destination The path in the particular uuNode where the volume shall be mounted.
      #   @return [UU::OS::UESURI] UESURI of the updated Volume.

      # @!method self.set_attributes(volume_uri, attributes)
      #   Sets attributes of a Volume. This method is a convenience
      #   shortcut for the {#set_attributes} instance method.
      #   @param volume_uri [String, UU::OS::UESURI] Volume URI.
      #   @param attributes [Hash] Volume attributes to update.
      #   @option attributes [String] name Volume name. May be +nil+.
      #   @option attributes [String] code Serves as identifier for a volume.
      #   @option attributes [String] source Identifier of the persistent storage on the cloud platform.
      #     The recommended format is +hexa32code+.
      #     E.g. name of File Share in Azure Files to use as backing storage.
      #   @option attributes [String] destination The path in the particular uuNode where the volume shall be mounted.
      #   @return [UU::OS::UESURI] UESURI of the updated Volume.

      # @!method delete(volume_uri)
      #   Deletes the specified Volume from the registry.
      #   @param volume_uri [String, UU::OS::UESURI] Volume URI.
      #   @return [void]

      # @!method self.delete(volume_uri)
      #   Deletes the specified Volume from the registry. This method is a
      #   convenience shortcut for the {#delete} instance method.
      #   @param volume_uri [String, UU::OS::UESURI] Volume URI.
      #   @return [void]

      # Returns a list of Volumes for the given Cloud. Command does
      # not return all Volumes, only a sublist. The list of returned Volumes
      # could be filtered using {https://docs.mongodb.org/manual/tutorial/query-documents/
      # Mongo Query}.
      #
      # @param cloud_uri [String, UU::OS::UESURI] Cloud URI.
      # @param criteria [Hash] Criteria for listing Volumes.
      # @option criteria [String] query Mongo Query to filter the result set.
      #   See {https://docs.mongodb.org/manual/tutorial/query-documents/ Mongo Query tutorial}.
      #   Allowed query filter document fields: name, code, driver, source, destination, appDeploymentUri
      # @option criteria [Fixnum] pageIndex Result set page index.
      # @option criteria [Fixnum] pageSize Result set page size.
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def get_volume_list(cloud_uri, criteria = {})
        @client.invoke(:getVolumeList, cloud_uri, parameters: criteria)
      end

      # Returns list of Volumes. This method is a
      # convenience shortcut for the {#get_volume_list} instance method.
      #
      # @see #get_volume_list
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def self.get_volume_list(cloud_uri, criteria = {})
        self.new(UU::OS::Security::Session.current_session).get_volume_list(cloud_uri, criteria)
      end

    end

  end
end
